/*---------------------------------------------------------------------------------------------------*\
|   Program Name: Age Stratification Tool -  Creates an age group variable consisting of custom,      |
|                                            user-specified groupings, with allowance for any mix of  | 
|                                            time units (i.e. years, quarters, months, weeks, and/or  |
|                                            days).                                                   |
|                                                                                                     |
|                                                                                                     |
|   Date: 06/28/2013                                                                                  |
|   Version: 2.0                                                                                      |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   This program creates an age group variable consisting of custom, user-specified groupings,        |
|   with allowance for any mix of time units.                                                         | 
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:                                                                                   |
|                  -SAS data file (.SAS7BDAT format) including START and END dates to calculate age   |
|                   for each observation and a text string defining the age stratifications           |
|                                                                                                     |
|                   or                                                                                |
|                                                                                                     |
|                  -SAS data file (.SAS7BDAT format) including a START date and a constant date       |
|                   parameter to calculate age for each observation and a text string                 |
|                   defining the stratifications                                                      | 
|                                                                                                     |
|            Note:                                                                                    |
|                  -The default value for the age stratification text string is                       |
|                   00-01 02-04 05-09 10-14 15-18 19-21 22-44 45-64 65-74 75+                         |
|                                                                                                     |
|   Program outputs:                                                                                  |
|                  -SAS data file (.SAS7BDAT format) containing the age group for each observation    |
|																									  |
|   Macro variable definitions:  																	  |
|  	INFILE    = Input file containing start date (required)					    					  |
|  	OUTFILE   = Output file  									                                      |
|  	STARTDT   = Start date to measure age				                                              |
|  	ENDDT     = End date to measure age								                                  |
|  	TIMESTRAT = Age groups for stratification 										                  |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
|                                                                                                     |
\*---------------------------------------------------------------------------------------------------*/

%macro MS_AGESTRAT(INFILE=,OUTFILE=,STARTDT=,ENDDT=,TIMESTRAT=);

	%global NBSTRAT; 

    *Converting constant end date to value in numeric if required;
	%if %index(&ENDDT.,/) %then %do;
		data _null_;
		Call Symput('ENDDT',put(input("&ENDDT",mmddyy10.),best12.));
		run;
		%put END DATE CONVERTED TO NUMERIC: &ENDDT;
	%end;
 
	*Setting default values for time text string (years);
	%if %str("&TIMESTRAT.")=%str("") %then %do;
		%let TIMESTRAT=00-01 02-04 05-09 10-14 15-18 19-21 22-44 45-64 65-74 75+;
	%end;
	
	*Counting time categories;
	%let NUMARG=0; 
	%do %while(%qscan(&TIMESTRAT,&NUMARG+1,%str( )) ne %str());
		%let NUMARG = %eval(&NUMARG+1);
	%end;
	
	%let NBSTRAT=&NUMARG.;

	*Defining each time category;
	data _NULL_;
	format AgeThresh $200. AgeTyp $200.;
	do i=1 to &NBSTRAT.;
		_agetyp=compress(scan("&TIMESTRAT.",i*2-1),'DWMQY','klu');
		_agetyp=TRANWRD(UPCASE(_agetyp),'Y','Years');
		_agetyp=TRANWRD(UPCASE(_agetyp),'D','Days');
		_agetyp=TRANWRD(UPCASE(_agetyp),'W','Weeks');
		_agetyp=TRANWRD(UPCASE(_agetyp),'Q','Quarters');
		_agetyp=TRANWRD(UPCASE(_agetyp),'M','Months');
			
		if _agetyp ne '' then 	AgeTyp=strip(AgeTyp)||" "||strip(_Agetyp);
		else AgeTyp=strip(AgeTyp)||' Years';
			
		AgeThresh=strip(AgeThresh)||" "||compress(scan("&TIMESTRAT.",i*2-1),'DWMQY','lu');
		if i=1 then MinAge=AgeThresh;
		if i=&NBSTRAT. then do;
			MaxAge=compress(scan("&TIMESTRAT.",i*2),'DWMQY','l')*1;
			if missing(MaxAge) then MaxAge=99999;
		end;
		output;
	end;
	call symput('AGETHRESH',AgeThresh);
	call symput('AGETYP',AgeTyp);
	call symput('MINAGE',MinAge);
	call symput('MAXAGE',MaxAge);
	run;

	*Echoing macro variable;
	%put _LOCAL_;
	
	*Assigning age categories;
	data &OUTFILE.;
	set &INFILE.;
	format AgeGroup $9. MinAgeDate MaxAgeDate mmddyy10.; 

	MinAgeDate = intnx(scan("&AGETYP.",1),&STARTDT.,&MINAGE.,'sameday');

    if &MAXAGE.=99999 then MaxAgeDate=intnx('Years',&STARTDT.,120,'sameday');
    else MaxAgeDate= intnx(scan("&AGETYP.",&NBSTRAT.),&STARTDT.,&MAXAGE.+1,'sameday')-1;

	if MinAgeDate <= &ENDDT. <= MaxAgeDate then do; 
		do i=&NBSTRAT. to 1 by -1;		    
			if &ENDDT. >= intnx(scan("&AGETYP.",i),&STARTDT.,scan("&AGETHRESH.",i),'sameday') then do;
				AgeGroup = scan("&TIMESTRAT.",i,' ');
				leave;
			end;
    	end;	      
	end;

	drop i MinAgeDate MaxAgeDate;
	run;

%mend MS_AGESTRAT;
