/*---------------------------------------------------------------------------------------------------*\
|   Program name: Macro to prepare the data to compute the Charlson Comorbidity Index (CCI)           |
|                                                                                                     |
|   Date: 08/06/2012                                                                                  |
|   Version: Beta 1                                                                                   |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   This program uses procedure and diagnosis claims data from the MSDD to create a dataset to        |
|   ultimately be used to run the ms_cci macro and compute the CCI                                    |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:                                                                                   |
|                  - One dataset containing Patid and reference date for which the CCI will be        |
|                    computed                                                                         |
|   Program outputs:                                                                                  |
|                  -SAS data file (.SAS7BDAT) to be used with the ms_cci Macro 	                      |
|																									  |
|   Macro variable definitions:  																	  |
|	PRIORDAYS = Number of days to define the prior lookup period to compute the Prior CCI (P)         |
|  	INDEXDAYS = Number of days to define the index lookup period to compute the Index CCI (I)         |
|  	INCLINDEX = variable to indicate whether to include the indexdate in the definition of the Index  |
|               lookup period.  Of note, if INCLINDEX >1, then the index lookup period will go        |
|               beyond the indexdate                                                                  |
|  	INFILE    = The name of the sas file containing the Patid and reference date to compute the CCI   |
|   INDEXDT   = The name of the variable containing the reference date                                |
|   PREOUTFILE= Name of the output dataset with only the neccesary data to compute the CCI            |
|   DIAGFILE  = Name of the file containing the diagnoses                                             |
|   PROCFILE  = Name of the file containing the procedures                                            |
|                                                                                                     |
|   Programming Notes:                                                                                |
|			       -The input file must contains PatId and a reference data.                          |
|                  -Eligibility criteria are assumed to have been checked (i.e., that patients are    |
|                   eligible for medical benefits in the PRIORDAYS-INCLINDEX interval                 |
|                  -PRIORDAYS>=INDEXDAYS                                                              |
|                  -for efficency reasons, only patients with claims in the PRIORDAYS-INCLINDEX       |
|                   interval will be in the CCI-ready file.  Consider re-merging with the INFILE      |
|                   or running the ms_post_cci macro to have the correct patient count at the end     |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
|                                                                                                     |
\*---------------------------------------------------------------------------------------------------*/


*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!;
*This macro needs the ms_wordcount tool;
*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!;    

%macro ms_cci_pre(PRIORDAYS=,INDEXDAYS=,INCLINDEX=, INFILE=, INDEXDT=, PREOUTFILE=,DIAGFILE=,PROCFILE=);	

/*****************************************/
/* Reduce data to selected patients      */
/* and prepare data for the comorb macro */
/*****************************************/

data &PREOUTFILE.(keep=PatId ICDDX IDXPRI HCPCS ICDPX);
	if 0 then set &INFILE.;
	declare hash ht (hashexp:16, dataset:"&INFILE.");
	ht.definekey('PatId');
	ht.definedata(ALL: 'YES');
	ht.definedone();

	*procedures;
	do until(eof2);
	retain patid;
	
	set &PROCFILE.(keep=patid adate Px Px_codetype) end=eof2;
		if ht.find()=0 then do;
			format IDXPRI $1.;
			IDXPRI='';
			if &INDEXDT.-&PRIORDAYS.   <= adate <= &INDEXDT.-&INDEXDAYS.-1  then IDXPRI='P';  
		    if &INDEXDT.-&INDEXDAYS.-1 <  adate < &INDEXDT.-(-1*&INCLINDEX.) then IDXPRI='I';
			if IDXPRI ne '' then do;
				HCPCS='     ';
				ICDPX='     ';
			    if PX_Codetype in ("C4") then do;
					HCPCS=PX;
					output;
				end;
			    else if PX_Codetype="09" then do;
					ICDPX=PX;
					output;
				end;
			end;
		end;
	end;

	*diagnosis;
	do until(eof1);
	retain patid;
		set &DIAGFILE.(keep=patid adate dx Dx_codetype) end=eof1;
		if ht.find()=0 then do;
			call missing(HCPCS,ICDPX);
			format IDXPRI $1.;
			IDXPRI='';
			if &INDEXDT.-&PRIORDAYS.   <= adate <= &INDEXDT.-&INDEXDAYS.-1  then IDXPRI='P'; 
		    if &INDEXDT.-&INDEXDAYS.-1 <  adate < &INDEXDT.-(-1*&INCLINDEX.) then IDXPRI='I';
			if IDXPRI ne '' and DX_Codetype="09" then output;
		end;
		rename DX=ICDDX;
	end;
	stop;
run;

proc sort nodupkey;by _ALL_;run;

%mend ms_cci_pre;
