/*---------------------------------------------------------------------------------------------------*\
|   Program name: Treatment Episode Reconciliation Tool.							  				  |														  
|                                                                                                     |
|   Date: 11/28/2012                                                                                  |
|   Version: 1.0                                                                                      |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   This program can use prescription drug claims data or inpatient claims data from the MSDD         |
|   and create treatment/hospital episodes using a user defined maximum allowable gap.				  |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:                                                                                   |
|                  -SAS data file with claims data            									      |
|                                                                                                     |
|   Program outputs:                                                                                  |
|                  -SAS data file (.SAS7BDAT format) containing the conciliated treatment episodes	  |
|																									  |
|   Macro variable definitions:  																	  |
|	INFILE		  = Name of the SAS data set containing the claims data								  |
|  	EPISODEGAP	  = Treatment episode maximum allowable gap	                                          |
|  	CLAIMEXPIREDT = Name of the claim expiry date variable in the INFILE 						      |
|  	OUTFILE		  = Name of the output file						                                      |
|                                                                                                     |
|   Programming Notes:                                                                                |
|			      - The input file must contain the PatId, Enr_Start and ADate (or RxDate) variables. |                                                   
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
|                                                                                                     |
\*---------------------------------------------------------------------------------------------------*/

%macro MS_CREATEEPISODES(INFILE=,EPISODEGAP=,CLAIMEXPIREDT=,OUTFILE=);

	*Determine if already sorted by PatId Enr_Start and ADate;
	%global SORTED;
                  
	%let RESORT=; 
	ods output attributes=attributes;
	proc contents data=&INFILE.;
	run;

	data _null_ ;
	set attributes;
	where label2=:"Sorted";
	call symput('SORTED',cvalue2);
	run;
	%put &SORTED.;

	%if &SORTED.=YES %then %do;
	
		ods output sortedby=sortedby;
		proc contents data=&INFILE.;
		run;

		data _null_;
		set sortedby;
		where label1=:"Sortedby";
		call symput('SORTEDBY',cvalue1);
		run;
		%put &SORTEDBY.;

		data _null_;
		length resort $3.;
		var=strip(compbl(upcase("&SORTEDBY.")));
		var2=strip(compbl(upcase("PatId Enr_Start ADate")));
		if var=var2 then resort="NO";
		else resort="YES";
		call symput ('RESORT',resort);
		run;
		%put RESORT=&RESORT.;

	%end;

	%if &SORTED.=NO or &RESORT.=YES %then %let SORT_AGAIN=YES;
	%else %let SORT_AGAIN=NO;

	*If not sorted then sort otherwise do not sort;
	%if &SORT_AGAIN.=YES %then %do;
	    proc sql noprint;
        create table _episodes as
        Select *
        from &INFILE.
        order by PatId, Enr_Start, ADate;
        quit;
	%end;
	%else %do;
	    data _episodes;
		set &INFILE.;
		run;
	%end;

	*Episode creation;
	data _episodes;
	set _episodes;
	by PatId;

	*Create Episode Indicator;
	LRunOutDate = lag(&CLAIMEXPIREDT.);  	
	LEnrStart = lag(Enr_Start);

	if first.PatId then do;
		LRunOutDate=.;		
        LEnrStart=.; 
		episode=1;
		gap=.;
	end;
	else do; 
		*Change episode if exceed gap or change enrollment;
		gap = ADate - LRunOutDate - 1;   		
		if gap > &EPISODEGAP. or lEnrStart ne Enr_Start then episode=episode+1; 
		
	end;

	retain episode;
	drop LRunOutDate LEnrStart gap;
	run;	

	*Calculate Episode Start and End dates. The bridging and same episode duplicate removal
	 should be done after running the macro;
	proc sql noprint;
	Create Table &OUTFILE. as
	Select 	epis.*,
       		min(ADate) as EpisodeStartDt format=mmddyy10.,
	  		max(ExpireDt) as EpisodeEndDt format=mmddyy10.
	From _episodes as epis  
	group by Patid, Episode;
	quit;
	
	proc datasets library = work nolist;
	delete _episodes attributes;
	quit;

	%put _LOCAL_;

%mend MS_CREATEEPISODES;
