/*---------------------------------------------------------------------------------------------------*\
|   Program name: Envelope algorithm execution requirement tool.									  |														  
|                                                                                                     |
|   Date: 11/28/2012                                                                                  |
|   Version: 1.0                                                                                      |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   This program scans a dataset containing the required care settings to determine if running the    |
|   envelope algorithm is required.  Logically, the envelope should be run as soon as one care        |
|   setting from the available list in the MSDD (OA, IP, ED, AV, IS) is not required.		          |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:                                                                                   |
|                  -SAS data file with required care setting values.			    				  |
|                                                                                                     |
|   Program outputs:                                                                                  |
|                  -Flag that indicates if running the envelope algorithm is required.   			  |
|																									  |
|   Macro variable definitions:  																	  |
|	INFILE		  = SAS dataset with required care setting values									  |
|   CARESETTING   = Name of the variable that contains the required care settings                     |
|  	OUTNEEDENVEL  = Name of the output macro variable 							                      |
|                                                                                                     |
|   Programming Notes:                                                                                |                                  
|                  -The input file must contain a caresetting variable                                |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
|                                                                                                     |
\*---------------------------------------------------------------------------------------------------*/

%macro MS_ENVELNEEDED(INFILE=,CARESETTING=,OUTNEEDENVEL=);
	
	%global &OUTNEEDENVEL.;
	%let &OUTNEEDENVEL.=0;

	%if %sysfunc(exist(&INFILE.))=1 %then %do;
    	proc contents data=&INFILE. noprint out=_tilt_ ;
    	data _null_ ;
    	set _tilt_ ;
    	call symput('nobs1',trim(left(put(nobs,15.)))) ;
    	run;
	%end;
	%else %do;
    	data _null_ ;
    	call symput('nobs1',input('0',15.)) ;
    	run;
 	%end;

	%let SOME=0;

	proc sort nodupkey data=&INFILE.(keep=&CARESETTING.)
                       out=_CS;
	by &CARESETTING.;
	run;

	data _null_;
	set _CS;    	
	
	&CARESETTING.=compress(&CARESETTING.,"'");

	*First determine whether the "all caresettings" option was always chosen ;
	if &CARESETTING. ne '' and not(indexw(&CARESETTING.,'OA') & 
                                   indexw(&CARESETTING.,'IP') & 
                                   indexw(&CARESETTING.,'ED') & 
                                   indexw(&CARESETTING.,'AV') & 
                                   indexw(&CARESETTING.,'IS'))
    then call symput('SOME',input('1',1.));   *this will trigger the envelope to run;
	else &CARESETTING.='';		
    run;
 	%put &SOME. &nobs1.;
	%put %eval(&nobs1.>0 and &SOME.=1);

  	%if %eval(&nobs1.>0 and &SOME.=1) %then %do;
		%let &OUTNEEDENVEL.=1;
	%end;
	%put &&&OUTNEEDENVEL.;

	proc datasets library = work nolist;
	delete _tilt_ _CS;
	quit;

%mend MS_ENVELNEEDED;
