/*---------------------------------------------------------------------------------------------------*\
|   Program name: Pharmacy Claims Extraction Tool                                                     |
|                                                                                                     |
|   Date: 11/28/2012                                                                                  |
|   Version: 1.0                                                                                      |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|    With a user-specified list of NDC codes, this program enables the extraction of pharmacy claims  | 
|    from the Mini-Sentinel Distributed Database (MSDD). This macro also gives the user the option    |
|    to remove duplicate claims (true duplicates).    										          |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:                                                                                   |
|                  -SAS data file with pharmacy claims               							      |
|                  -SAS data file with NDC code list                                                  |
|                                                                                                     |
|   Program outputs:                                                                                  |
|                  -SAS data file with selected pharmacy claims                                       |
|																									  |
|   Macro variable definitions:  																	  |
|	INFILE		= Name of pharmacy claims dataset								                      |
| 	NDCINFILE	= Name of the NDC lookup dataset						                              |
|  	REMOVEDUP	= Indicator for the removal of duplicate claims							              |
|  	OUTFILE		= Output dataset with pharmacy claims									              |
|                                                                                                     |
|   Programming Notes:                                                                                |
|				   -The INFILE must have a codetype value that begins with RX or have a 	          |
| 					proc variable with the value 0 (please see documentation for details)             |    
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
|                                                                                                     |
\*---------------------------------------------------------------------------------------------------*/

%macro MS_GETDRUGS(INFILE=,NDCINFILE=,REMOVEDUP=,OUTFILE=);
		
	*Check if duplicates should be removed;
	%let distinct=;
	%if %upcase(%str("&REMOVEDUP.")) eq %str("YES") %then %do;		
		%let distinct= distinct ;		
	%end;	

	*Check if column proc is present and split codetype if not;
	%let procexist=0;

	data _null_; 
	dsname="&NDCINFILE.";	
	varname='proc';
	dsid=open(dsname); 
	if dsid > 0 then do; 
		call symput('procexist',put(varnum(dsid,varname),best.)); 
	end;
	rc=close(dsid); 
	run;
	%put &procexist.;

	%if %eval(&procexist.=0) %then %do;
		data _codelist (keep=CodeType Code length);
		set &NDCINFILE.;
		length=length(Code);	
		if CodeType in:('RX') then do; 
			codetype=compress(codetype,'RX'); 		
			output; 		
		end;			
		run;
	%end;
	%else %do;
		*Reduce NDCINFILE to minimum rows and columns for performance gain;
		data _codelist (keep=CodeType Code length);
		set &NDCINFILE. (where=(proc=0));			
		length=length(code);		
		run;
	%end;

	proc sort nodupkey data=_codelist;
	by CodeType Code;
	run;

	* Check for NDC9 and NDC11 extraction;
	%let ISCode9=0;
    %let ISCode11=0;
    data _null_;
    set _codelist;
    if length=9 then call symput("ISCode9",1);
    if length=11 then call symput("ISCode11",1);
    run;
    %put &ISCode9;
    %put &ISCode11;	

    * Extract 9 digit Codes claims;
	%if %eval(&ISCode9.>0) %then %do;		     	
    	proc sql noprint;
        create table _drugs9 as
        select &distinct. Dispensing.*,
               0 as Proc,
			   9 as length
        from _codelist as CodeList,
             &INFILE. as Dispensing
        where substr(Dispensing.Ndc,1,9) = CodeList.code;
        quit;
    %end;
    
    * Extract 11 digit Codes claims;
    %if %eval(&ISCode11.>0) %then %do;
    	proc sql noprint;
        create table _drugs11 as
        select &distinct. Dispensing.*,
               0 as Proc,
			   11 as length
        from _codelist as CodeList,
             &INFILE. as Dispensing
        where substr(Dispensing.Ndc,1,11) = CodeList.code;
        quit;
	%end;

	* Generate OUTFILE and clean temporary datasets;
	%if %eval(&ISCode9.>0 and &ISCode11.>0) %then %do;
		data &OUTFILE.;
		set _drugs9 _drugs11;
		run;
		proc datasets library = work nolist;
		delete _drugs9 _drugs11 _codelist;
		quit; 
	%end;
	%if %eval(&ISCode9.>0 and &ISCode11.=0) %then %do;
		data &OUTFILE.;
		set _drugs9;
		run;
		proc datasets library = work nolist;
		delete _drugs9 _codelist;
		quit;
	%end;
	%if %eval(&ISCode9.=0 and &ISCode11.>0) %then %do;
		data &OUTFILE.;
		set _drugs11;
		run;
		proc datasets library = work nolist;
		delete _drugs11 _codelist;
		quit;
	%end;	

%mend MS_GETDRUGS;
