/*---------------------------------------------------------------------------------------------------*\
|   Program Name: Medical Claims Extraction Tool                                                      |
|                                                                                                     |
|                                                                                                     |
|   Date: 11/28/2012                                                                                  |
|   Version: 1.0                                                                                      |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   With a user specified list of diagnosis and/or procedure codes, this program extracts diagnosis   |
|   and/or procedure claims from the MSDD. The user also has the option to remove duplicate claims.   |
|   Please note that if XXX is specified in the list of diagnosis and/or procedure codes, the         |
|   algorithm will extract all claims with a code starting by XXX. Consequently, specifying           |
|   the code XXXYY when the code XXX is identified will result in a duplicate if a XXXYY coded claim  |
|   is encountered notwithstanding the option to remove duplicates.                                   | 
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:                                                                                   |
|                  -SAS data file (.SAS7BDAT format) containing the procedure claims                  |
|                  -SAS data file (.SAS7BDAT format) containing the diagnosis claims                  |
|                  -SAS data file (.SAS7BDAT format) with the procedure/diagnosis codes list          |
|                                                                                                     |
|   Program outputs:                                                                                  |
|                  -SAS data file (.SAS7BDAT format) containing the extracted medical claims.         |
|																									  |
|   Macro variable definitions:  																	  |
|	PROCINFILE = Procedure claims SAS dataset							                              |
|	DIAGINFILE = Diagnosis claims SAS dataset							                              |
|	LISTINFILE = List of diagnosis and/or procedure codes											  |
|	REMOVEDUP  = Flag for the removal of duplicate claims						            		  |
|	OUTFILE    = Output file with exctracted medical claims                     					  |	
|                                                                                                     |
|   Programming Note:                                                                                 |                                                                                                                                                     |
|                 -The compilation of the MS_MAKEMACVAR program is a prerequisite to compiling        |
|                  and running the MS_GETMEDS program                                                 | 
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
|                                                                                                     |
\*---------------------------------------------------------------------------------------------------*/

*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!;
*This macro needs the MS_MAKEMACVAR tool;
*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!;

%macro MS_GETMEDICAL(PROCINFILE=,DIAGINFILE=,LISTINFILE=,REMOVEDUP=,OUTFILE=);
	
	* Check if duplicates should be removed;
	%let distinct=;

	%if %upcase(%str("&REMOVEDUP.")) eq %str("YES") %then %do;		
		%let distinct= distinct ;		
	%end;	
	
	*Check if column proc is present and split codetype if not;
	%let procexist=0;

	data _null_; 
	dsname="&LISTINFILE.";	
	varname='proc';
	dsid=open(dsname); 
	if dsid > 0 then do; 
		call symput('procexist',put(varnum(dsid,varname),best.)); 
	end;
	rc=close(dsid); 
	run;
	%put &procexist.;

	%if %eval(&procexist.=0) %then %do; 
		data _diaglist _proclist;
		set &LISTINFILE.;
		Code=compress(Code,'.');
		length=length(code);
		if CodeType in:('DX') then do; 
			codetype=compress(upcase(codetype),'DX'); 
			proc=-1; 
			output _diaglist; 		
		end;
		if CodeType in:('PX') then do;
			codetype=compress(upcase(codetype),'PX');
			proc=1; 
			output _proclist;
		end;
		keep CodeType Code proc length;
		run;
	%end;
	%else %do;
		*Reduce LISTINFILE to minimum rows and columns for performance gain;
		data _diaglist _proclist;;
		set &LISTINFILE.;
		Code=compress(Code,'.');
		length=length(code);
		if proc=-1 then output _diaglist;
		if proc=1 then output _proclist;
		keep CodeType Code proc length;
		run;
	%end;	

	proc sql noprint;
		select count(1) into :pnobs from _proclist; 	
		select count(1) into :dnobs from _diaglist;
	quit;

	* Some procedures required;
	%if %eval(&pnobs.>0) %then %do;
		* Create variable of procedures;
		proc sort nodupkey data=_proclist;
		by CodeType Code proc;
		run;

		%MS_MAKEMACVAR(INFILE=_proclist,VARCOLUMN=Code,PREFIX="'",SUFFIX="'",SEPARATOR=" ",OUTVAR=VAR_PROC);

		* Prefiltering to remove dot in codes and keep only applicable codes for performance gain;
		proc sql noprint;
			Create Table _procextract(rename=PX_Codetype=Codetype) as
			select &distinct. *, 
				   compress(PX,'.') as Code				   
			from &PROCINFILE. (where=(compress(PX,'.') in:(&VAR_PROC.)));
		quit;

		proc sql noprint;
			Create Table _procedures as
			Select prctb.*, 
				   proclist.proc, 
				   proclist.length, 
				   1 as RxSup, 
				   0 as RxAmt, 
				   'X' as PDX 	
			From _procextract (drop=PX) as prctb,
				 _proclist as proclist
			Where prctb.CodeType = proclist.codetype 
			      and substr(prctb.Code,1,proclist.length) = proclist.code;			     
		quit;
	%end;

	* Some diagnosis required;
	%if %eval(&dnobs.>0) %then %do;		
		*Create variable of diagnosis;
		proc sort nodupkey data=_diaglist;
		by CodeType Code proc;
		run;

		%MS_MAKEMACVAR(INFILE=_diaglist,VARCOLUMN=Code,PREFIX="'",SUFFIX="'",SEPARATOR=" ",OUTVAR=VAR_DIAG);

		* Prefiltering to remove dot in codes and keep only applicable codes for performance gain;
		proc sql noprint;
			Create Table _diagextract(rename=DX_Codetype=Codetype) as
			select &distinct. *, 
				   compress(DX,'.') as Code				   
			from &DIAGINFILE. (where=(compress(DX,'.') in:(&VAR_DIAG.)));
		quit;

		proc sql noprint;
			Create Table _diagnosis as
			Select diagtb.*, 
				   diaglist.proc,
				   diaglist.length	
			From _diagextract (drop=DX) as diagtb,
			     _diaglist as diaglist
			Where diagtb.CodeType = diaglist.codetype
			      and substr(diagtb.Code,1,diaglist.length) = diaglist.code;
		quit;
	%end;

	* Creation of output file and temporary datasets clean up;
	%if %eval(&pnobs.> 0 and &dnobs.>0) %then %do;
		data &OUTFILE.;
		set _procedures _diagnosis;
		run;
		proc datasets library = work nolist;
		delete _procedures _diagnosis _procextract _diagextract _proclist _diaglist;
		quit; 
	%end;
	%if %eval(&pnobs.> 0 and &dnobs.=0) %then %do;
		data &OUTFILE.;
		set _procedures;
		run;
		proc datasets library = work nolist;
		delete _procedures _procextract _proclist;
		quit;
	%end;
	%if %eval(&pnobs.=0 and &dnobs.>0) %then %do;
		data &OUTFILE.;
		set _diagnosis;
		RxSup=.;
		RxAmt=.;
		run;
		proc datasets library = work nolist;
		delete _diagnosis _diagextract _diaglist;
		quit;
	%end;	

%mend MS_GETMEDICAL;
