/*---------------------------------------------------------------------------------------------------*\
|   Program name: Macro to count medical visits to assess patient complexity 	                      |
|                                                                                                     |
|   Date: 08/12/2012                                                                                  |
|   Version: Beta 1                                                                                   |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   This program uses encounter dataset from the MSDD to count the number of distinct visits from a   |
|   reference date and within a lookback period.  Any encounter  type (or group of) can be specified. |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:       																			  |
|				   - One dataset containing the Encounter data										  |
|                  - One dataset containing Patid and reference date from which the count will be     |
|                    computed                                                                         |
|   Program outputs:                                                                                  |
|                  -SAS data file (.SAS7BDAT) with the Patid and the number of visits (group)         |
|																									  |
|   Macro variable definitions:  																	  |
|                                                                                                     |
|  ENCOUNTERFILE= The name of the sas dataset containing encounter data							      |
|      INDEXFILE= The name of the sas dataset containing the Patid and reference date to compute      |
|                 the number of visits                                                                |
|        INDEXDT= The name of the variable containing the reference date in INDEXFILE                 |
|   CARESETTINGS= The encounter type (caressetting) to be used in the count macro                     |
|      PRIORDAYS= Number of days to define the lookback period to compute the counts                  |
|      INCLINDEX= Whether to include the reference data in the lookback period (1=yes 0=no)           |
|        CSSTRAT= The grouping of counts that the user wishes to apply.  Groups need to be separated  |
|                 by a space, intervals defined using "-" and use "+" if the last group is open ended.|
|        OUTFILE= Name of the output dataset containing the counts                                    | 
|                                                                                                     |
|   Programming Notes:                                                                                |
|			       -The INDEXFILE input file must contains PatId and a reference data.                |
|			       -As with all MS Programs, this macro assumes that the encounter table has been     |
|                   defined according to the MSCDM                                                    |
|                  -Eligibility criteria are assumed to have been checked (i.e., that patients are    |
|                   eligible for medical benefits in the PRIORDAYS-INCLINDEX interval                 |
|                  -PRIORDAYS>=INDEXDAYS)                                                             |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
|                                                                                                     |
\*---------------------------------------------------------------------------------------------------*/

*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!;
*This macro Requires the ms_wordcount tool;
*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!; 		

%macro ms_util(ENCOUNTERFILE=,INDEXFILE=,INDEXDT=,CARESETTINGS=,PRIORDAYS=,INCLINDEX=,CSSTRAT=,OUTFILE=);	

/*************************************/
/* Reduce data to selected patients  */
/* and timeframe                     */
/*************************************/

	data CS(keep=Patid Adate);;
	if 0 then set &INDEXFILE.;
	declare hash ht (hashexp:16, dataset:"&INDEXFILE.");
	ht.definekey('PatId');
	ht.definedata(ALL: 'YES');
	ht.definedone();

	* Encounters;
	do until(eof2);
		set &ENCOUNTERFILE.(keep=patid adate EncType) end=eof2;
		if ht.find()=0 then do;
			if &INDEXDT.-&PRIORDAYS. <= adate < &INDEXDT.-(-1*&INCLINDEX.) then do;
					if indexw(compress("&CARESETTINGS.","'"),EncType) then output;
			end;
		end;
	end;
	stop;
	run;

	%MS_WORDCOUNT(TXTVECTOR=&CSSTRAT., WORDCOUNT=NUMGRP);

	* Create format for strata;
	%let ALLVAR=;
	data _null;
		format FMTSTR $300.;
		if &NUMGRP=0. then FMTSTR='0-high = "0+"';
		else do i=1 to &NUMGRP.;
			word=tranwrd(scan("&CSSTRAT.",i,' '), "+", "-high")||' = "'|| scan("&CSSTRAT.",i,' ')||'" ';
			FMTSTR=strip(FMTSTR)||" "||trim(left(word));
			output;
		end;
		call symput("FMTSTR",FMTSTR);
	run;
	%put &FMTSTR.;

	PROC FORMAT;
	   VALUE CSSTRAT
		&FMTSTR.
	;
	run;

	* Create output table;
	proc sql noprint;
	create table &OUTFILE. as 
    select PatId,
	       count(distinct Adate) as ExactNumVisit format CSSTRAT.
	from CS
	group by PatId
	order by PatId;
	quit;

	data &OUTFILE.;
	merge &INDEXFILE.(in=a keep=PatId) &OUTFILE.;
	by PatId;
	if a;
	if ExactNumVisit=. then ExactNumVisit=0;
	run;

	data &OUTFILE.;
	retain patid NumVisit ExactNumVisit;
	format ExactNumVisit best.;
	set &OUTFILE.;
	NumVisit=put(ExactNumVisit, CSSTRAT.);
	run;

	proc freq data=&OUTFILE.;
	tables NumVisit;	
	run;

	proc freq data=&OUTFILE.;
	tables NumVisit;
	run;

	proc datasets library = work nolist;
	delete CS;
	quit;

%mend ms_util;	
