****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: 00_master.sas  
*
* Created (mm/dd/yy): 11/01/2013
* Last modified: 11/01/2013
* Version: 1.0
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*  Mini-Sentinel : Propensity Score Matching PROMPT (Prospective Routine Observational Monitoring Program Tool) 
*
*	 The Initial Developer of the Original Code is the 							  
*	 Division of Pharmacoepidemiology and Pharmacoeconomics, 					  
*	 Department of Medicine, Brigham and Women's Hospital and 					  
*	 Harvard Medical School					  									  
*																				  
*	 Module Leads: Joshua J Gagne, Shirley V Wang, Sebastian Schneeweiss			  
*	 Programmer:   Shirley V Wang												  
*	 Uses Pharmacoepidemiology Toolbox macros, created by Jeremy Rassen	
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Mini-Sentinel Coordinating Center
*  info@mini-sentinel.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials	   Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*             mm/dd/yy
*
***************************************************************************************************;

/* System options */
options mprint 	linesize=190 pagesize=50 compress=yes reuse=no symbolgen ERRORS=25
				sastrace=',,,d' sastraceloc=saslog NOSTSUFFIX NOQUOTELENMAX
				FORMCHAR="|----|+|---+=|-/\<>*";
				
				
*-----------------------------------
* Assign STANDARD Parameters
*-----------------------------------;

/* Define Data Partner and Site IDs according to table below */
%let DPID = [enter-dpid];
%let SITEID = [enter-siteid];
 
 /*-------------------------------------------------------------------*\
 ||                                                                   ||
 ||  DATA PARTNERS                                 DPID      SITEID   ||
 ||  -----------------------------------------------------------------||
 ||   Healthcore (one site)                         HC         OS     ||
 ||   Humana (one site)                             HU         OS     ||
 ||   Aetna (one site)                              AE         OS     ||
 ||   Optum (one site)                              OP         OS     ||
 ||   Vanderbilt University (one site)              VB         OS     ||
 ||   HMORN                                                           ||
 ||      Group Health Cooperative                   HM         GHC    ||
 ||      Fallon Community Health Plan               HM         MPCI   ||   
 ||      Henry Ford Health System                   HM         HFHS   || 
 ||      Lovelace Health System                     HM         LCF    || 
 ||      Marshfield Clinic                          HM         MCRF   ||
 ||      HealthPartners                             HM         HPRF   ||
 ||      Harvard Pilgrim Health Care                HM         HPHC   ||
 ||   Kaiser Permanente                                               ||         
 ||      Kaiser DCC                                 KP         DCC    ||
 ||      Kaiser Permanente Colorado                 KP         CO     ||
 ||      Kaiser Permanente Georgia                  KP         GA     ||
 ||      Kaiser Permanente Hawaii                   KP         HI     ||
 ||      Kaiser Permanente Northern California      KP         NC     ||
 ||      Kaiser Permanente Northwest                KP         NW     ||
 ||      Kaiser Permanente Mid Atlantic             KP         MA     ||
 \*-------------------------------------------------------------------*/
   
/* Define MSOC-assigned request ID */
%let REQUESTID = [enter-requestid];


/* Define MSCDM table names */
%let enrtable=enrollment;
%let demtable=demographic;
%let distable=dispensing;
%let enctable=encounter;
%let diatable=diagnosis;
%let proctable=procedure;
%let deatable=death;
%let vittable=vitals; 


/* Assign paths in subsections a through g below */
/* IMPORTANT NOTE: END OF PATH SEPARATORS ARE REQUIRED!! */
/*		
			Windows-based platforms:  
				"\", e.g. "C:\user\sas\" and not "C:\user\sas";					
			Unix-based platforms:           										
				"/", e.g."/home/user/sas/" and not "/home/user/sas";	
*/

/* a. Data in MSCDM Format */		  		
libname indata "/your-path/mscdm" access=readonly;

/* b. Directory containing input files */  	
%let infolder = /your-path/inputfiles/;
libname infolder "&infolder.";

/* c. Directory to which files will be written, for sharing outside of the Data Partner */
%let msoc= /your-path/msoc/;
libname msoc "&MSOC.";

/* d. Directory to which files will be written, to be kept locally at the Data Partner */
%let dplocal= /your-path/dplocal/;
libname dplocal "&DPLocal.";

/* e. Directory to which temporary hd-PS files will be written */
%let temphdps = /your-path/hdps/temphdps/; 
libname temphdps "&tempHDPS";

/* f. Directory containing Pharmacoepidemiology Toolbox Macros */
/* IMPORTANT NOTE: PATHNAME CANNOT HAVE SPACES!!*/
%let toolbox = /your-path/hdps/toolbox/; 

/* SAS %include statements; no editing needed */
%include "&toolbox.hdmacros_v214.sas";
%include "&toolbox.matching_v214.sas";
%include "&toolbox.java_utils_v214.sas";
%let jar = &toolbox.pharmacoepi_v214.jar;
%javautils_InitClasspathUpdate;
%javautils_AddToClasspath(&jar);
%toolbox_Start(%QUOTE(&jar));
%include "&toolbox.table_creator_v214.sas";
%include "&toolbox.ms_cci_pre.sas";
%include "&toolbox.ms_cci.sas";
%include "&toolbox.ms_cci_post.sas";
%include "&toolbox.ms_util.sas";
%include "&toolbox.ms_lockdata.sas";
%include "&toolbox.ms_wordcount.sas";
%include "&toolbox.ms_episoderec.sas";
%include "&toolbox.ms_agestrat.sas";
%include "&toolbox.ms_lockdata.sas";


/* g. Directory containing suite of Cohort Matching Modular Programs */
%let query = /your-path/sasprograms/;

/* SAS %include statements; no editing needed */
*Dates of sequential monitoring periods;
%include "&query.01_monitoring.sas";
*Modular Program 3, version 5;
%include "&query.02_mp3_5.0.sas";
*Modular Query Adjustment;
%include "&query.03_modular_adjustment_query.sas";
*Modular Query Display and Diagnostics;
%include "&query.04_table_creator.sas";

*-----------------------------------
* Define Prospective Macro
*-----------------------------------;
%MACRO PROSPECTIVE(LOOK = );
	%monitoringperiod();

	/* Create output dataset for timing of query */
	%if &look = 1 %then %do;
	DATA msoc.&DPID.&SITEID._&REQUESTID._TIMING;
		length COHORTRUNTIME $150.;
		length ADJUSTMENTRUNTIME $150.;
		length PREDEFINEDRUNTIME $150.;
		length HDPSRUNTIME $150.;
		length MATCHRUNTIME $150.;
		length TABLETIME $150.;
		length TABLEAUTIME $150.;
		length MONITORINGPERIOD $2.;
		length DATES $30.;
		do i = 1 to 30;
		COHORTRUNTIME = " ";
		ADJUSTMENTRUNTIME = " ";
		PREDEFINEDRUNTIME = " ";
		HDPSRUNTIME = " ";
		MATCHRUNTIME = " ";
		TABLETIME = " ";
		TABLEAUTIME = " ";
		MONITORINGPERIOD = put(i, 2.);
		DATES = " ";
		OUTPUT;
		end;
		drop i;
	RUN;
	%end;


*-----------------------------------
* Assign Parameters And Call Modular Program 3 To Create New User Cohorts With Defined Eligibility Criteria
*-----------------------------------;
%let enrolgap = ; /** Enrollment gap for censoring follow up **/
%let queryfile = ; /** MP3 QUERYFILE **/
%let incqueryfile = ; /** MP3 INCQUERYFILE **/
%let queryeventfile = ; /** MP3 QUERYEVENTFILE **/
%let inceventfile = ; /** MP3 INCEVENTFILE **/
%let condfile = ; /** MP3 PRE_EXISTING CONDITION FILE */
%let agestrat = ; /** SPECIFY AGE STRATA **/
%let ccifile = ; /** MP3 CCI FILE **/
%let utilfile = ; /** MP3 UTIL FILE **/
%let bmikidsfile = ; /** MP3 BMI KIDS FILE **/
%let outtablesfile = ; /** MP3 OUT TABLES FILE **/

%MODULARPROGRAM3(
	REQUESTID=&requestid.,
	ENROLGAP=&enrolgap.,
	QUERYFROM=&startfollowup.,
	QUERYTO=&enddate.,
	QUERYFILE=&queryfile.,
	INCQUERYFILE=&incqueryfile.,
	QUERYEVENTFILE=&queryeventfile.,
	INCEVENTFILE=&inceventfile.,
	CONDFILE=&condfile.,
	AGESTRAT=&agestrat.,
	CCIFILE=&ccifile.,
	UTILFILE=&utilfile.,
	BMIKIDSFILE=&bmikidsfile.,
	OUTTABLESFILE=&outtablesfile.
);


*-----------------------------------
* Call Adjustment Query:
* Creates Predefined Covariates, Comorbidity Score, Health Service Utilization Indicators, 
* Creates As Treated And ITT Event Indicators, Follow Up Time 
* Runs hd-PS, Matching
*-----------------------------------;
%global datastartcheck useHOImacro covariatewindow covariatecondition covariaterx drugclass comorbidscore ITT;

%let datastartcheck = ; /** Name of .sas7bdat file with start dates of relevant tables for DP **/
%let useHOImacro = ; /** Specify whether you want to add outcome defined by
															HOI Macro instead of MP3 (leave blank if you want to use MP3 defined outcome)
															Notes: 
															1. If HOI macro event is desired, this macro must
															be identified and %included in section 3.g above 
															2. WASHPER and WASHTYP are extracted from QUERYEVENTFILE **/
%let covariatewindow = ; /** SPECIFY LENGTH (in DAYS) of covariate assessment window **/
%let covariatecondition = ; /** Name of .sas7bdat with covariate condition dx, category, and subgroup indicator **/
%let covariaterx = ; /** Name of .sas7bdat with covariate drug ndc, category, and subgroup indicator **/
%let covariateproc = ; /** Name of .sas7bdat with covariate condition px, category, and subgroup indicator **/
%let comorbidscore = ; /** Name of .sas7bdat for creating COMBINED COMORBIDITY SCORE **/
%let drugclass = ; /** Name of .sas7bdat with NDC-DRUGCLASS CROSSWALK **/
%let itt = ; /** SPECIFY FOLLOW UP (in DAYS) for Intention to Treat (ITT)
									!!THIS NUMBER MUST BE LESS THAN OR EQUAL TO EXPEXTPER IN QUERYFILE!! 
									If left blank, the default will be to produce event indicator 
									and follow up time for As Treated analyses.
									Analysis will be censored for death or disenrollment. **/


	*-----------------------------------
	* Optional - Change hd-PS And Matching Parameters From Default Values
	*-----------------------------------;
	%global covariates_considered covariates_selected ranking health_service_intensity zero_cell_corr;

	%let covariates_considered = ; /** specify number of covariates to consider for inclusion in hdPS model
														  					default is 100 **/
	%let covariates_selected = ; /** specify number of covariates from pool of considered covariates to keep in hdPS model 
														  					default is 200 or number new users of study drug with smallest N 
														  					(whichever is smaller) **/
	%let ranking = ; /** specify which method to use for selection of covariates
												default is exposure association based selection**/
	%let health_service_intensity = ; /** default is 1 (yes) **/
	%let zero_cell_corr = ; /** default is 1 (yes) **/
	%let nz = ; /** specify whether netezza is available, default is 0 (no) **/

*-----------------------------------
* Call COVARIATEADJUSTMENT Macro
*-----------------------------------;
%COVARIATEADJUSTMENT();

*-----------------------------------
*  Call Diagnostics Query To Create Figures And Tables For MSOC               
*-----------------------------------;
* Treatment Groups;
%let grp1 = ;	/* Names of primary and referent exposure groups in STUDYCLASS */
%let grp2 = ;	/* Must be exactly the same as in STUDYCLASS variable (capitalization matters!) */

%let templatetable = ; /* Name of excel file with input template for table 1s */
%matchtables(); /* Run macro to create table 1s and PS distribution figures */

%MEND;


*-----------------------------------
* Define Prospective Look Number
*  	Runs Programs 01-04 for sequential monitoring periods start/end dates defined in 01_monitoring.sas
*		Edit the %PROSPECTIVE(LOOK = ) macro call below to reflect the sequential monitoring period start/end dates.
*-----------------------------------;
%PROSPECTIVE(LOOK = ); 
