****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: 03_modular_adjustment_query.sas  
*
* Created (mm/dd/yy): 11/01/2013
* Last modified: 11/01/2013
* Version: 1.0
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*  Mini-Sentinel : Propensity Score Matching PROMPT (Prospective Routine Observational Monitoring Program Tool) 
*
*	 The Initial Developer of the Original Code is the 							  
*	 Division of Pharmacoepidemiology and Pharmacoeconomics, 					  
*	 Department of Medicine, Brigham and Women's Hospital and 					  
*	 Harvard Medical School					  									  
*																				  
*	 Module Leads: Joshua J Gagne, Shirley V Wang, Sebastian Schneeweiss			  
*	 Programmer:   Shirley V Wang												  
*	 Uses Pharmacoepidemiology Toolbox macros, created by Jeremy Rassen	
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Mini-Sentinel Coordinating Center
*  info@mini-sentinel.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials	   Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*             mm/dd/yy
*
***************************************************************************************************;


/**********************************************************************************************************
ADJUSTMENT QUERY - CREATE PREDEFINED and HDPS COVARIATES, MATCH
**********************************************************************************************************/

%MACRO COVARIATEADJUSTMENT();

/*********************************************************************
	START LOG
**********************************************************************/
proc printto log="&MSOC.&DPID.&SITEID._&REQUESTID._adjusted_&look..log" new;
RUN;
	
%put "MODULARPROGRAM ADJUSTMENT WITH HDPS";

/**************************************************************
	Import QUERYFILE and QUERYEVENTFILE
***************************************************************/

%MACRO IMPORTFILES(var1,var2,var3);
   %IF %INDEX(%UPCASE("&VAR1."),CPORT) %THEN %DO;

proc cimport infile="&infolder.&VAR1." library=infolder memtype=data;
run;

   %END;
   %LET &VAR2.=%SUBSTR(&VAR1.,1,%INDEX(%UPCASE(&VAR1.),.)-1);
   %LET VAR1=%SUBSTR(&VAR1.,1,%INDEX(%UPCASE(&VAR1.),.)-1);

   data _&var2.;
   set infolder.&var1.;
   group = compress(trim(left(group)));
   group = translate(group,'_','-',
                           '_','.',
                           '_',',',
                           '_','%',
                           '_','$',
                           '_','!',
                           '_','*',
                           '_','&',
                           '_','#',
                           '_','@');
   keep &var3.;
run;

%MEND;
%IMPORTFILES(&QUERYFILE.,QUERYFILE,_ALL_);   
%IMPORTFILES(&QUERYEVENTFILE.,QUERYEVENTFILE,_ALL_); 

/**************************************************************
	Check for User Input Errors 
**************************************************************/
%put Check for User Input Errors;

	%macro check();

		/* requestid cannot be left black */
		%if &requestid = %str() %then %do;
		    %put error: REQUESTID assigned from MSOC must be entered as parameter;
	    	%abort;
	  	%end;

		/* COMORBIDSCORE cannot be left blank */
		%if &comorbidscore = %str() %then %do;
		    %put error: comorbidscore must specify name of a .sas7bdat file for creating combined comorbidity score;
	    	%abort;
	  	%end;

		/* If As Treated Analysis then ITT must be blank  
		   If Intention to Treat Analysis, then ITT should = number of days for ITT follow up
			
		   HOWEVER!!!!
		   If using MP3 defined outcome then !!THIS NUMBER MUST BE LESS THAN OR EQUAL TO EXPEXTPER IN QUERYFILE!!
				ITT <=EXPEXTPER<=EPISODEGAP<=WASHPER in queryfile  
		   If using flexible HOI macro then do not need to worry about EXPEXTPER being <= ITT */ 
		%if &itt ne %str() %then %do;
			PROC SQL;
				select count(distinct q.group) into :numgroup
				from _queryfile as q;
				select min(expextper)
				into :ExpExtPer1-:ExpExtPer%eval(&numgroup)
				from _queryfile as q
				group by q.group;
				select min(episodegap)
				into :EpisodeGap1-:EpisodeGap%eval(&numgroup)
				from _queryfile as q
				group by q.group;
				select min(Washper)
				into :Wash1-:Wash%eval(&numgroup)
				from _queryfile as q
				group by q.group;
			QUIT;
			%do c = 1 %to &numgroup;
				%if &useHOImacro ne 1 %then %do;
					%if &&ExpExtPer&c <&itt or &&EpisodeGap&c <&&ExpExtPer&c  or &&Wash&c < &&EpisodeGap&c %then %do;
					%put error: ITT must be blank for As Treated analysis;
					%put error: ITT must be le EXPEXTPER and EXPEXTPER must be le EPISODEGAP in QUERYFILE for ITT analysis;
					%put error: EPISODEGAP must be < WASHPER in QUERYFILE for ITT analysis;	    	
					%abort;
					%end; * if conditions are not met;
				%end; * when not using HOI macro;
				%if &useHOImacro = 1 %then %do;
					%if  &&EpisodeGap&c <&&ExpExtPer&c or &&Wash&c < &&EpisodeGap&c %then %do;
					%put error: ITT must be blank for As Treated analysis;
					%put error: EXPEXTPER must be le EPISODEGAP in QUERYFILE;
					%put error: EPISODEGAP must be < WASHPER in QUERYFILE;	    	
					%abort;
					%end; * if conditions are not met;
				%end; *when using HOI Macro;
		  	%end; * c to numgroup;
		%end; * if ITT ne str();

		/* Dataset with START dates for available data in DP cannot be left blank */
		%if &datastartcheck = %str() %then %do;
		    %put error: Dataset with START dates of available data in relevant tables cannot be left blank;
	    	%abort;
	  	%end;

		/* Make sure there is enough lag between start of DATA availability and start
		   of surveillance to assess new user status and have an appropriate look back 
		   window for covariate assessment */

		PROC SQL; 
			select max(washper) into :maxwashper from _queryfile; 
		QUIT;

		PROC SQL;
			select min(startdate) into :startdata
			from infolder.&datastartcheck
			where upcase(dpid) =upcase("&dpid.") and upcase(siteid) = upcase("&siteid.");
		QUIT;


		DATA _null_;
			sf = "&startfollowup";
			month1=substr(sf, 1, 2);
			day1=substr(sf, 4, 2);
			year1=substr(sf, 7, 4);
			startsurveillance = mdy(input(month1,2.),input(day1,2.),input(year1,4.));
			sd = "&startdata";
			call symput("lag", startsurveillance-sd);
		RUN; 

		%if &lag< &maxwashper or &lag< &covariatewindow %then %do;
			%put error: Surveillance begins on &startfollowup however the first rx claim in dataset on &startdata;
			%if &lag< &maxwashper %then %put error: Surveillance begins without sufficient lag (washper) from start of available data for assessment of new user status;
			%if &lag< &covariatewindow %then %put error: Surveillance begins without sufficient lag (washper) from start of available data for covariate assessment window;
		%abort;
		%end;

	/* Is netezza available? */
	%global bulkload bulkunload nz_or_work;
		%if &nz = %str() or &nz = 0 %then %do;
			%let bulkload = ;
			%let bulkunload = ;
			%let nz_or_work = ;
		%end;
		%if &nz = 1 %then %do;
			%let bulkload = (BULKLOAD = YES errlimit = 25);
			%let bulkunload = (BULKUNLOAD = YES);
			%let nz_or_work = %quote(indata.);
		%end;
	
	/* Initialize timing macros */
		%global covariatetime predefinedtime hdpstime matchingtime;

		%let covariatetime = %str();
		%let predefinedtime = %str();
		%let hdpstime = %str();
		%let matchingtime = %str();

	%mend check;
	%check();

/*********************************************************************
 	CLEAR INDATA LIBRARIES
**********************************************************************/
%put CLEAR INDATA LIBRARY IF NETEZZA IS AVAILABLE;

%macro cleanup();

%if &nz = 1 %then %do;

	%let datatables = e_indicator mastertable mastertableunique demo uniquedemo demo2
			&covariatecondition covarconditions count2 covarconditions2 covarind covarind2 covarind3
			rx covmed covmed2 covmed_ind covmed_ind2 covmed_ind3
			&covariateproc covarproc covarproc2 covprocind covprocind2 covprocind3
			deathdates dischargedead enrollend censor newuser&look temp2 drugclass
			diag proc rx_hd nume rcount 
			comorbid comorbid2 comorbid3 temp3;

	%let wordcount=%sysfunc(countw(&datatables));
	%put &wordcount;

	%do i = 1 %to &wordcount;
		%let table =%scan(&datatables,&i,' ');
			%if %sysfunc(exist(&nz_or_work.&table.)) %then %do;
			proc sql;
			drop table &nz_or_work.&table.;
			quit;
			%end;
	%end;

%end; /* if nz = 1 */

%mend;

%cleanup();

/**************************************************************
 Ensure all character macro parameters are lowercase
***************************************************************/

	%let REQUESTID=%LOWCASE(&REQUESTID.);
	%let QUERYFILE=%LOWCASE(&QUERYFILE.);
	%let COVARIATECONDITION=%LOWCASE(&COVARIATECONDITION.);
	%let COVARIATEPROC=%LOWCASE(&COVARIATEPROC.);
	%let COMORBIDSCORE=%LOWCASE(&COMORBIDSCORE.);

/********************************************************
   GET OVERALL ADJUSTMENT START TIME
 *******************************************************/

        DATA _NULL_;
        temp=DATETIME();
        call symput('START',temp);
        call symput('STARTDATE',put(datepart(temp),date9.));
        call symput('STARTTIME',put(timepart(temp),time4.));
        RUN;

/********************************************************
	Keep only relevant variables from MP3 
	If patient initiated more than one drug on same day, drop patient 
*********************************************************/

DATA keepvars; 
SET DPLocal.&DPID.&SITEID._&REQUESTID._MasterTable_&look. 
	(rename = (group = studyclass)
	 rename = (episodestartdt = indexdt));
keep patid studyclass indexdt episodeenddt eventdt;
RUN;

/*******************************************************************************************
 	Ensure that patients do not have a dispensing from multiple GROUPs 
   	on the index date for their new user treatment episode 
   (MP3 output dataset will not include information on dispensations from other GROUPS 
	if they were not part of a valid episode)
********************************************************************************************/
	* put all codes in any GROUP into macro variable list;

	data drugcodes;
		set infolder.&queryfile (rename = (group = druggroup));
		keep druggroup code;
	run;

	PROC SQL; 

		create table druggroupdates as
		select patid, ndc, rxdate, druggroup 
		from 
			(select * 
			 from indata.&DISTABLE 
			 where ndc in(select code from infolder.&queryfile)
			 ) as dis
		inner join drugcodes as dg 
		on dis.ndc = dg.code;

		create table sameday as
		select distinct k.patid, 1 as multexp 
		from keepvars as k
		inner join druggroupdates as d 
		on k.patid = d.patid
			and k.indexdt = d.rxdate
		where studyclass ne druggroup;

		create table temp as
		select k.*
		from keepvars as k
		left join sameday as s
		on k.patid = s.patid
		where multexp ne 1;

	QUIT; 

/*******************************************************
	CREATE BINARY INDICATOR FOR exposure 
********************************************************/
	%global prior;
	%let prior = %eval(&look-1); 

%macro wrapper();
	%if %sysfunc(exist(dplocal.&DPID.&SITEID._&REQUESTID._matched_&prior.)) = 0 %then %do;
		PROC SQL;
			create table size as
			select studyclass, count(*) as count
			from temp
			group by studyclass
			order by count desc;

			create table &nz_or_work.e_indicator &bulkload as
			select monotonic()-1 as exposure, studyclass
			from size;
		QUIT;
	%end;
	%if %sysfunc(exist(dplocal.&DPID.&SITEID._&REQUESTID._matched_&prior.)) %then %do;
		PROC SQL;
			create table &nz_or_work.e_indicator &bulkload as
			select distinct exposure, studyclass
			from dplocal.&DPID.&SITEID._&REQUESTID._scores_1;
		QUIT;
	%end;
%mend;
%wrapper();  


/** GET START TIME FOR CREATING PREDEFINEDS, DEMOGRAPHICS, FOLLOW UP **/

        DATA _NULL_;
        temp=DATETIME();
        call symput('STPRE',temp);
        call symput('STDATEPRE',put(datepart(temp),date9.));
        call symput('STTIMEPRE',put(timepart(temp),time4.));
        RUN;

PROC SQL;

        /** ADD BINARY exposure INDICATOR TO MASTERTABLE from MP3 **/
        create table &nz_or_work.mastertable &bulkload AS
        select M.*, exposure 
		from temp as M, &nz_or_work.E_INDICATOR as E
        where M.studyclass = E.studyclass;

        /** IF PATIENT APPEARS as "NEW USER" MORE THAN ONCE, KEEP FIRST **/
        create table &nz_or_work.mastertableunique &bulkload as (
        select MT.* from &nz_or_work.mastertable as MT,
        (
                select patid, MIN(indexdt) as MINindexdt
                from &nz_or_work.mastertable
                group by patid
        ) as nodups
         where MT.patid = nodups.patid and MT.indexdt = nodups.MINindexdt
        );

/*********************************************************************
 GET DEMOGRAPHIC CHARACTERISTICS FOR NEW USERS IDENTIFIED IN QUARTER
 IF PATIENT HAS MULTIPLE VALUES FOR DEMOGRAPHIC CHARACTERISTICS, DROP THEM
**********************************************************************/

        create table &nz_or_work.demo &bulkload  AS
        select distinct newuser.*, BIRTH_DATE, SEX, RACE
        from indata.&demtable as DEM
        inner join &nz_or_work.mastertableunique as newuser
        on DEM.patid = NEWUSER.patid;

		create table &nz_or_work.uniquedemo &bulkload as
		select d.*, count(*) as numdemo
		from &nz_or_work.demo as d
		group by d.patid;

  QUIT; 

        /** CALCULATE AGE and AGE STRATA SPECIFIED BY MSOC **/
  %ms_agestrat(	infile=&nz_or_work.uniquedemo(where=(numdemo=1)),
             	outfile=&nz_or_work.demo2 &bulkload,
	     		startdt=birth_date,
	     		enddt=indexdt,
	     		timestrat=&agestrat.);


/* Create indicators:
		How many new users identified during this monitoring period? 
		How many exposure groups identified during this monitoring period? 				*/
	%macro skip();

		%global numNewExpgroups numNewPatients;
		%let numNewExpgroups = 0;
		%let numNewPatients = 0;

		%if &look = 1 or %sysfunc(exist(dplocal.&DPID.&SITEID._&REQUESTID._matched_&prior.)) = 0 %then %do;

			PROC SQL;

				select count(distinct studyclass) into :numNewExpgroups
				from &nz_or_work.demo2;
				select count(distinct patid) into :numNewPatients
				from &nz_or_work.demo2;

				create table newpatients as
				select patid, indexdt
				from &nz_or_work.demo2;

			QUIT;

			/* If no new users identified, abort program */
			%if &numNewPatients = 0 %then %do;
				%put error: No new users within calendar time window and age boundaries of this scenario have been identified;
				%abort;
			%end;

		%end; * if first monitoring period;

		%if &look > 1 or %sysfunc(exist(dplocal.&DPID.&SITEID._&REQUESTID._matched_&prior.)) %then %do;

			PROC SQL;

				select count(distinct studyclass) into :numNewExpgroups
				from &nz_or_work.demo2
				where patid not in (select patid from dplocal.&DPID.&SITEID._&REQUESTID._matched_&prior.);
				select count(distinct patid) into :numNewPatients
				from &nz_or_work.demo2
				where patid not in (select patid from dplocal.&DPID.&SITEID._&REQUESTID._matched_&prior.);

				create table newpatients as
				select patid, indexdt
				from &nz_or_work.demo2
				where patid not in (select patid from dplocal.&DPID.&SITEID._&REQUESTID._matched_&prior.);

			QUIT;
	
		/* If no new users have been identified in any monitoring period prior and none have been
		   identified in this monitoring period, abort program  */
			%if  %sysfunc(exist(dplocal.&DPID.&SITEID._&REQUESTID._matched_&prior.)) and &numNewPatients = 0 %then %do;
				%put error: No new users within calendar time window and age boundaries of this scenario have been identified;
				%abort;
			%end;
		%end; * if not first monitoring period; 

	%mend;

	%skip();

/****************************************************************************
       CREATE PREDEFINED COVARIATE CONDITIONS
*****************************************************************************/

%macro globalmacro();
	%global 	numcondition numc nocovarcondition 
				numrx numr nocovarrx
				numproc nump nocovarproc
				matchid;
	%do i = 1 %to 99;
		%global covar&i covmed&i covproc&i subdx&i subpx&i subrx&i;
	%end;
	%let matchid = 	PREDEFINEDPSMATCHID1_1-PREDEFINEDPSMATCHID1_3 
					HDPSPREDEFINEDMATCHID1_1-HDPSPREDEFINEDMATCHID1_3
				    HDPSONLYMATCHID1_1-HDPSONLYMATCHID1_3
					PREDEFINEDPSMATCHID100_1-PREDEFINEDPSMATCHID100_3 
					HDPSPREDEFINEDMATCHID100_1-HDPSPREDEFINEDMATCHID100_3
				    HDPSONLYMATCHID100_1-HDPSONLYMATCHID100_3;
%mend; 
%globalmacro();

%macro covariatedx(); 

	/***********************************************************************
	 CREATE DIAGNOSIS DATASET FOR NEW USERS
	************************************************************************/
	PROC SQL;

	      create table &nz_or_work.diag &bulkload AS
	      select m.patid, adate, upcase(compress(DX, '.')) as dx, indexdt, enctype, dx_codetype, pdx,
				 SUBSTR(upcase(compress(dx.DX, '.')),1,3) as D3, 
				 SUBSTR(upcase(compress(dx.DX, '.')),1,4) as D4, 
				 SUBSTR(upcase(compress(dx.DX, '.')),1,5) as D5
	      from indata.&diatable as dx
	      inner join newpatients as m
		  on dx.patid = m.patid
	      where indexdt-&COVARIATEWINDOW<=ADATE and ADATE<indexdt;

	QUIT;

	%if %sysfunc(exist(infolder.&COVARIATECONDITION)) %then %do;

		%let nocovarcondition = 0;

	 PROC SQL;

	      /** BRING IN CONDITION FILE **/
	      create table &nz_or_work.&COVARIATECONDITION &bulkload AS
	      select * from infolder.&COVARIATECONDITION;

	      /** GET RELEVANT DIAGNOSES from CONDITION FILE and MSCDM DIAGNOSIS FILE **/
	      create table &nz_or_work.COVARCONDITIONS &bulkload as
	      select patid, adate, dx.dx, studydiag
	      from &nz_or_work.diag as dx
	      right outer join &nz_or_work.&covariatecondition as codx
		  on upcase(compress(dx.DX, '.'))= upcase(compress(codx.DX, '.'))
			and dx.dx_codetype = codx.dx_codetype;

	/** RECODE CONDITIONS TO BINARY INDICATORS and LABEL WITH STUDYDIAG **/

	      create table count AS
	      select DISTINCT STUDYDIAG as distinct_diag, subgroup
	      from infolder.&COVARIATECONDITION
		  ORDER BY STUDYDIAG asc;

	      select COUNT(DISTINCT distinct_diag)
	      into :numcondition
	      from count;

	      select distinct_diag
	      into :COVAR1-:COVAR99
	      from count
		  order by distinct_diag;

		  select subgroup
	      into :subdx1-:subdx99
	      from count;

	 QUIT;

		  DATA COUNT2;
	      SET COUNT;
	      NUMBER = _n_; TEMP = 1;
	      CALL SYMPUT("numc", strip(put(&numcondition, 3.)));
	      RUN;

	 PROC SQL;

	      create table &nz_or_work.covarconditions2 &bulkload AS
	      select c.*, number, temp
		  from &nz_or_work.covarconditions as C
		  inner join count2 as c2
	      on C.studydiag = c2.distinct_diag;

	      create table &nz_or_work.covarind &bulkload AS
	      select  DISTINCT M.patid, studydiag, number, temp
	      from &nz_or_work.covarconditions2 as C 
		  right outer join newpatients as M
	      on C.patid = M.patid;

	 QUIT;

	 	* if first monitoring period with new users identified then do;
		%if &look = 1 or %sysfunc(exist(dplocal.&DPID.&SITEID._&REQUESTID._matched_&prior.)) = 0 %then %do;

		DATA &nz_or_work.covarind3 &bulkload;
		      SET &nz_or_work.covarind;
		      by patid;
		      ARRAY COVAR(&NUMC) COVAR1-COVAR&NUMC;
		      RETAIN COVAR1-COVAR&NUMC;
		      DO i = 1 to &numc;
		      IF FIRST.patid THEN COVAR(i) = 0;
		      IF NUMBER = i THEN COVAR(i) = 1;
		      END;
			  patid1 = patid;
		      IF LAST.patid THEN OUTPUT;
		      KEEP patid1 COVAR1-COVAR&NUMC;
	    RUN; 
		
		%end;

	 	* if not the first monitoring period where new users were identified, 
		  then bring in covariates from prior monitoring periods;
		%if %sysfunc(exist(dplocal.&DPID.&SITEID._&REQUESTID._matched_&prior.)) = 1 %then %do;

		DATA &nz_or_work.covarind2 &bulkload;
		      SET &nz_or_work.covarind;
		      by patid;
		      ARRAY COVAR(&NUMC) COVAR1-COVAR&NUMC;
		      RETAIN COVAR1-COVAR&NUMC;
		      DO i = 1 to &numc;
		      IF FIRST.patid THEN COVAR(i) = 0;
		      IF NUMBER = i THEN COVAR(i) = 1;
		      END;
			  patid1 = patid;
		      IF LAST.patid THEN OUTPUT;
		      KEEP patid1 COVAR1-COVAR&NUMC;
	    RUN; 

	  	DATA &nz_or_work.covarind3 &bulkload;
			 set &nz_or_work.covarind2
				dplocal.&DPID.&SITEID._&REQUESTID._matched_&prior. (rename = (patid = patid1));
		     KEEP patid1 COVAR1-COVAR&NUMC;
	    RUN; 
		
		%end; 

	%end; * If the input dataset exists;

	%if %sysfunc(exist(infolder.&COVARIATECONDITION)) = 0 %then %do; 
		%let nocovarcondition = 1;
		%let numc = 1;

		PROC SQL;
		create table &nz_or_work.covarind3 &bulkload as
		select patid as patid1
		from &nz_or_work.demo2;
		QUIT;

	%end; *if the input dataset does not exist;

%mend; 


/***************************************************************
 CREATE PREDEFINED COVARIATE MEDICATION INDICATORS
****************************************************************/
%macro covariaterx();
 
	/** Identify relevant NDC for New Users from the MSCDM file **/
	PROC SQL;
	      create table &nz_or_work.RX &bulkload AS
	      select NEWUSER.patid, RXDATE, NDC, indexdt
		  from indata.&distable as disp
	      inner join newpatients as NEWUSER
	      on DISP.patid = NEWUSER.patid  
		  where (indexdt-&COVARIATEWINDOW)<=RXDATE and RXDATE<indexdt and RXSUP>0;
	QUIT;

	%if %sysfunc(exist(infolder.&COVARIATERX)) %then %do;

		%let nocovarrx = 0;

		/* Extract 9 or 11 digit codes */
		 %LET ISCode9=0;
	  	 %LET ISCode11=0;

		data _null_;
		   set infolder.&covariaterx;
		   if length(ndc) = 9 then call symput("ISCode9",1);
		   if length(ndc) = 11 then call symput("ISCode11",1);
		run;

	   %PUT &ISCode9;
	   %PUT &ISCode11;

	 PROC SQL;

	      /** Create categories based on file from the MSOC **/
	      create table &nz_or_work.covmed &bulkload AS
	      select patid, indexdt, RXDATE, RX.NDC, DRUG_CAT 
		  from &nz_or_work.RX as RX
	      inner join INFOLDER.&covariaterx(where=(length(ndc) = 9)) as DRUGCLASS	
	      on substr(RX.NDC, 1, 9) = DRUGCLASS.NDC
		  union
		  select patid, indexdt, RXDATE, RX.NDC, DRUG_CAT 
		  from &nz_or_work.RX as RX
		  inner join INFOLDER.&covariaterx(where=(length(ndc) = 11)) as DRUGCLASS	
	      on substr(RX.NDC, 1, 11) = DRUGCLASS.NDC;
	
	      /** CREATE MACROS WITH THE NUMBER and NAME OF COVARIATE MEDICATION CATEGORIES **/
	      create table count AS
	      select DISTINCT DRUG_CAT as distinct_rx, subgroup
	      from INFOLDER.&covariaterx
		  ORDER BY DRUG_CAT asc;

	      select COUNT(DISTINCT distinct_rx)
	      INTO :numrx
	      from count;

	      select distinct_rx
	      INTO :COVMED1-:COVMED99
	      from count
		  order by distinct_rx;

		  select subgroup
	      into :subrx1-:subrx99
	      from count;

	 QUIT;

	      DATA COUNT2;
	      SET COUNT;
	      NUMBER = _n_; TEMP = 1;
	      CALL SYMPUT("numr",strip(put(&numrx, 3.)));
	      RUN;

	 PROC SQL;

	     create table &nz_or_work.covmed2 &bulkload AS
	     select * from &nz_or_work.covmed as C, count2 as c2
	     where C.drug_cat = C2.distinct_rx
		 ORDER BY DRUG_CAT;

	     create table &nz_or_work.covmed_ind &bulkload AS
	     select  DISTINCT M.patid, drug_cat, number, temp
	     from &nz_or_work.covmed2 as C 
		 right outer join newpatients as M
	     on C.patid = M.patid;

	 QUIT;

		* if the first monitoring period where new users identified then do;
		%if &look = 1 or %sysfunc(exist(dplocal.&DPID.&SITEID._&REQUESTID._matched_&prior.)) = 0 %then %do;

	     DATA &nz_or_work.covmed_ind3 &bulkload;
		     SET &nz_or_work.covmed_ind;
		     by patid;
		     ARRAY COVMED(&numr) COVMED1-COVMED&numr;
		     RETAIN COVMED1-COVMED&NUMR;
		     DO i = 1 to &numr;
		     IF FIRST.patid THEN COVMED(i)= 0;
		     IF NUMBER = i THEN COVMED(i)= 1;
		     END;
			 patid2 = patid;
		     IF LAST.patid THEN OUTPUT;
		     KEEP patid2 COVMED1-COVMED&NUMR;
	     RUN;

		%end;

		* if not the first monitoring period, bring in covariates from prior monitoring periods;
		%if %sysfunc(exist(dplocal.&DPID.&SITEID._&REQUESTID._matched_&prior.)) = 1  %then %do;

		 DATA &nz_or_work.covmed_ind2 &bulkload;
		     SET &nz_or_work.covmed_ind;
		     by patid;
		     ARRAY COVMED(&numr)   COVMED1-COVMED&numr;
		     RETAIN COVMED1-COVMED&NUMR;
		     DO i = 1 to &numr;
		     IF FIRST.patid THEN COVMED(i)= 0;
		     IF NUMBER = i THEN COVMED(i)= 1;
		     END;
			 patid2 = patid;
		     IF LAST.patid THEN OUTPUT;
		     KEEP patid2 COVMED1-COVMED&NUMR;
	     RUN;

	  	DATA &nz_or_work.covmed_ind3 &bulkload;
			 set &nz_or_work.covmed_ind2
				dplocal.&DPID.&SITEID._&REQUESTID._matched_&prior. (rename = (patid = patid2));
		     KEEP patid2 COVMED1-COVMED&NUMR;
	    RUN; 
		
		%end; 

	%end; * If input file exists;

	%if %sysfunc(exist(infolder.&COVARIATERX)) = 0 %then %do; 
		%let nocovarrx = 1;
		%let numr = 1;

		PROC SQL;
		create table &nz_or_work.covmed_ind3 &bulkload as
		select patid as patid2
		from &nz_or_work.demo2;
		QUIT;

	%end; * If input file does not exist;
%mend;


%macro covariatepx();

	/***********************************************************************
	 CREATE PROCEDURE DATASET FOR NEW USERS
	************************************************************************/
	PROC SQL;

	      create table &nz_or_work.proc &bulkload AS
	      select newuser.patid, adate, compress(PX.PX, '.') as px, indexdt, enctype, px_codetype
	      from indata.&proctable as px
	      inner join newpatients as NEWUSER
		  on px.patid = NEWUSER.patid
	      where indexdt-&COVARIATEWINDOW<=ADATE and ADATE<indexdt;

	QUIT;

	%if %sysfunc(exist(infolder.&COVARIATEPROC)) %then %do;

		%let nocovarproc = 0;

	 PROC SQL;

	      /** BRING IN CONDITION FILE **/
	      create table &nz_or_work.&COVARIATEPROC &bulkload AS
	      select * from infolder.&COVARIATEPROC;

	      /** GET RELEVANT DIAGNOSES from CONDITION FILE and MSCDM DIAGNOSIS FILE **/
	      create table &nz_or_work.COVARPROC &bulkload as
	      select patid, adate, compress(PX.PX, '.') as px, studyproc
	      from &nz_or_work.proc as px
	      right outer join &nz_or_work.&covariateproc as COPX 
		  on upcase(compress(px.PX, '.')) = upcase(compress(copx.PX,'.'))
		  	and upcase(px.PX_Codetype) = upcase(copx.PX_Codetype);

	/** RECODE CONDITIONS TO BINARY INDICATORS and LABEL WITH STUDYDIAG **/
	      create table count AS
	      select DISTINCT STUDYPROC as distinct_proc, subgroup
	      from infolder.&COVARIATEPROC
		  ORDER BY STUDYPROC asc;

	      select COUNT(DISTINCT distinct_proc)
	      into :numproc
	      from count;

	      select distinct_proc
	      into :COVPROC1-:COVPROC99
	      from count
		  order by distinct_proc;

		  select subgroup
	      into :subpx1-:subpx99
	      from count;

	 QUIT;

	      DATA COUNT2;
	      SET COUNT;
	      NUMBER = _n_; TEMP = 1;
	      CALL SYMPUT("nump", strip(put(&numproc, 3.)));
	      RUN;

	 PROC SQL;

	      create table &nz_or_work.covarproc2  &bulkload AS
	      select c.*, number, temp 
		  from &nz_or_work.covarproc as C
		  inner join count2 as c2
	      on C.studyproc = c2.distinct_proc;

	      create table &nz_or_work.covprocind &bulkload AS
	      select  DISTINCT M.patid, studyproc, number, temp
	      from &nz_or_work.covarproc2 as C 
		  right outer join newpatients as M
	      on C.patid = M.patid;

	 QUIT;

	 	* if the first monitoring period where new users identified then do;
		%if &look = 1 or %sysfunc(exist(dplocal.&DPID.&SITEID._&REQUESTID._matched_&prior.)) = 0 %then %do;

	      DATA &nz_or_work.covprocind3 &bulkload;
		      SET &nz_or_work.covprocind;
		      by patid;
		      ARRAY COVPROC(&NUMP) COVPROC1-COVPROC&NUMP;
		      RETAIN COVPROC1-COVPROC&NUMP;
		      DO i = 1 to &nump;
		      IF FIRST.patid THEN COVPROC(i) = 0;
		      IF NUMBER = i THEN COVPROC(i) = 1;
		      END;
			  patid3 = patid;
		      IF LAST.patid THEN OUTPUT;
		      KEEP patid3 COVPROC1-COVPROC&NUMP;
	      RUN;

		%end;

		* if not the first monitoring period, bring in covariates from prior monitoring periods;
		%if %sysfunc(exist(dplocal.&DPID.&SITEID._&REQUESTID._matched_&prior.)) = 1 %then %do;
	
	      DATA &nz_or_work.covprocind2 &bulkload;
		      SET &nz_or_work.covprocind;
		      by patid;
		      ARRAY COVPROC(&NUMP) COVPROC1-COVPROC&NUMP;
		      RETAIN COVPROC1-COVPROC&NUMP;
		      DO i = 1 to &nump;
		      IF FIRST.patid THEN COVPROC(i) = 0;
		      IF NUMBER = i THEN COVPROC(i) = 1;
		      END;
			  patid3 = patid;
		      IF LAST.patid THEN OUTPUT;
		      KEEP patid3 COVPROC1-COVPROC&NUMP;
	      RUN;

	  	DATA &nz_or_work.covprocind3 &bulkload;
			 set &nz_or_work.covprocind2
				dplocal.&DPID.&SITEID._&REQUESTID._matched_&prior. (rename = (patid = patid3));
		     KEEP patid3 COVPROC1-COVPROC&NUMP;
	    RUN; 
		
		%end; 

	%end; * If input dataset exists;

	%if  %sysfunc(exist(infolder.&COVARIATEPROC)) = 0 %then %do; 

		%let nocovarproc = 1;
		%let nump = 1;

		PROC SQL;
		create table &nz_or_work.covprocind3 &bulkload as
		select patid as patid3
		from &nz_or_work.demo2;
		QUIT;

	%end; * If input dataset does not exist;

%mend; 

%macro keep_sub_ind_for_msoc();

	%global subgroup_dx subgroup_px subgroup_rx;

	/* Diagnosis based subgroups */
	%let subgroup_dx = ;
	%do i = 1 %to &numc;
		%if &&subdx&i = 1 %then %let subgroup_dx = &subgroup_dx covar&i;
	%end;
	%put &subgroup_dx;
	/* Procedures based subgroups */
	%let subgroup_px = ;
	%do i = 1 %to &nump;
		%if &&subpx&i = 1 %then %let subgroup_px = &subgroup_px covproc&i;
	%end;
	%put &subgroup_px;
	/* Drug based subgroups */
	%let subgroup_rx = ;
	%do i = 1 %to &numr;
		%if &&subrx&i = 1 %then %let subgroup_rx = &subgroup_rx covmed&i;
	%end;
	%put &subgroup_rx;
%mend;

/****************************************************************
MACRO LIST PREDEFINED COVARIATE CONDITIONS, PROCEDURES, RX 
*****************************************************************/
%macro confounderlist();

	%global covardxlist covarrxlist covarpxlist;
	%let covardxlist = COVAR1-COVAR&numc;
	%let covarrxlist = COVMED1-COVMED&numr;
	%let covarpxlist = COVPROC1-COVPROC&nump;

	/** IF NO COVARIATE CONDITIONS OR NO COVARIATE RX OR NO COVARIATE PROCEDURES DEFINED THEN DO **/
	%IF &nocovarcondition = 1 %then %do;
		%let covardxlist = ;
		%end;

	%IF &nocovarrx = 1 %then %do;
		%let covarrxlist = ;
		%end;

	%IF &nocovarproc = 1 %then %do;
		%let covarpxlist = ;
		%end;
%mend;


/*************************************************************************
 GET DATE FOR END OF FOLLOW UP - OUTCOME, CENSOR (END ENROLL OR DEATH) 
 *************************************************************************/

	 /* DID PATIENT DIE BEFORE END OF TREATMENT EPISODE EXTENSION? 
		Note - use all identified patients, not just new patients in this monitoring period
		in case of new data regarding censoring, events

		Defensive coding - 
			Keep all death dates, discharged dead dates - 
			If date is before index date, drop patient 				*/

	 /* CHECK TO SEE IF DEATH TABLE EXISTS */
	%if %sysfunc(exist(indata.&deatable)) %then %do;

		 PROC SQL;
	     create table &nz_or_work.deathdates &bulkload AS
	     select M.patid, indexdt, min(deathdt) as deathdt
	     from &nz_or_work.demo2 as M 
		 inner join indata.&deatable as D
	     on M.patid = D.patid
	     where upcase(confidence) = "E"
		 group by m.patid;
		 QUIT;

	%end;

	%if %sysfunc(exist(indata.&deatable)) = 0 %then %do;

	%put Note: death data are not available;

		DATA &nz_or_work.deathdates &bulkload;
		set &nz_or_work.demo2;
		length deathdt 4.;
		patid = " ";
		indexdt = .;
		deathdt = .;
		if patid ne " ";
		keep patid indexdt deathdt;
		RUN;

	%end;

	/* WAS PATIENT DISCHARGED DEAD? */
		PROC SQL;

			create table &nz_or_work.dischargedead &bulkload AS 
			select distinct M.patid, indexdt, min(ddate) as discharged_dead 
			from &nz_or_work.demo2 as M 
			inner join indata.&enctable(where=(upcase(discharge_status) = "EX")) as E 
			on M.patid = E.patid 
			group by m.patid; 

		QUIT;

	/* DID PATIENT DISENROLL PRIOR TO END OF EPISODE EXTENSION PERIOD? */
		PROC SQL;
		    create table &nz_or_work.enrollend &bulkload AS
		    select distinct e.*
		    from &nz_or_work.demo2 as M 
			inner join indata.&enrtable as E
		    on M.patid = E.patid;
		QUIT;

		* reconcile episode gaps;
		%MS_EPISODEREC (INFILE=&nz_or_work.enrollend,
	                    OUTFILE=ms_episoderec,
	 	           		MEDCOV=Y,	
		           		DRUGCOV=Y,        
		           		ENRSTART=Enr_Start,
	        	        ENREND=Enr_End,
	        	        ENROLGAP=&ENROLGAP.,
	        	        REMOVEDUP=Y);

	PROC SQL;

		create table minenrend as
		select m.patid, min(EEnd) as minenr_end
		from ms_episoderec as e
		inner join &nz_or_work.demo2 as m
		on e.patid = m.patid
		where EStart<=indexdt and indexdt <=EEnd
		group by e.patid;

     	create table &nz_or_work.censor &bulkload as
     	select M.patid as patid4, M.indexdt, episodeenddt, 
				COALESCE(deathdt, &endfollowup.d) as deathdt, 
				COALESCE(discharged_dead, &endfollowup.d) as discharged_dead, 
				minenr_end
     	from &nz_or_work.demo2 as M
 	 	left join &nz_or_work.deathdates as D
     	on M.patid = D.patid
		left join &nz_or_work.dischargedead as DD
		on M.patid = DD.patid
     	left join minenrend as E
     	on M.patid = E.patid;

	QUIT;

	/* IF AT LEAST 1 NEW USER IDENTIFIED THEN CREATE PREDEFINED COVARIATES */
	%if &numNewPatients>0 %then %do;

			%covariatedx;
			%covariaterx;
			%covariatepx;
			%keep_sub_ind_for_msoc();
			%confounderlist();

		PROC SQL;
			create table &nz_or_work.newuser&look &bulkload AS
			select   d.patid, studyclass, eventdt, d.indexdt, exposure, 
					 deathdt, discharged_dead, minenr_end, d.episodeenddt,
			         ((d.indexdt-birth_date)/365) as AGE, agegroup as AGE_CAT, SEX, RACE, COVMED.*, COVAR.*, COVPROC.*  
			from &nz_or_work.demo2 as D
			inner join &nz_or_work.COVARIND3 as COVAR
			on d.patid = COVAR.patid1
			inner join &nz_or_work.COVMED_IND3 as COVMED
			on d.patid = COVMED.patid2
			inner join &nz_or_work.COVPROCIND3 as COVPROC
			on d.patid = COVPROC.patid3
			inner join &nz_or_work.CENSOR as C
			on d.patid = C.patid4
			/* Defensive coding - Make sure patient does not have death prior to index */ 
			where d.indexdt<=deathdt and d.indexdt<=discharged_dead 
			order by studyclass, patid;
		QUIT;

	%end;

	/* 	IF NO NEW USERS IDENTIFIED, THEN UPDATE FOLLOW UP/CENSORING VARIABLES 
		FOR PREVIOUSLY IDENTIFIED USERS */
	%if &numNewPatients = 0 and %sysfunc(exist(dplocal.&DPID.&SITEID._&REQUESTID._matched_&prior.)) %then %do;

		DATA priorcovariates;
			set dplocal.&DPID.&SITEID._&REQUESTID._matched_&prior. (rename = (patid = patid5));
			keep patid &covardxlist &covarrxlist &covarpxlist;
		RUN;

		PROC SQL;
			create table &nz_or_work.newuser&look &bulkload (drop = patid5) AS
			select   d.patid, studyclass, eventdt, d.indexdt, exposure, 
					 deathdt, discharged_dead, minenr_end, d.episodeenddt,
			         ((birth_date-d.indexdt)/365) as AGE, agegroup as AGE_CAT, SEX, RACE, p.* 
			from &nz_or_work.demo2 as D
			inner join &nz_or_work.CENSOR as C
			on d.patid = C.patid4
			inner join priorcovariates as p
			on d.patid = p.patid5
			/* Defensive coding - Make sure patient does not have death prior to index */ 
			where d.indexdt<=deathdt and d.indexdt<=discharged_dead 
			order by studyclass, patid;
		QUIT;

	%end;

		/** TIMING TO CREATE PREDEFINED COVARIATES **/
        DATA _NULL_;
        temp=DATETIME();
        seconds=temp-&stpre.;
        hours=int(seconds/3600);
        minutes=int((seconds-hours*3600)/60);
        seconds2=int((seconds-hours*3600-minutes*60));
        call symput('STOP',temp);
        call symput('hours',put(hours,4.0));
        call symput('minutes',put(minutes,2.0));
        call symput('seconds',put(seconds2,2.0));
        RUN;

		%global predefinedtime;
        %let predefinedtime = &hours. h &minutes. m &seconds. s ;

 /* IF ANALYSIS REQUIRES MORE COMPLEX HOI MACRO DEFINED OUTCOME
	THEN RUN HOI MACRO												*/ 

	%if &useHOImacro = 1 %then %do;
		%hoi(cohort = &nz_or_work.newuser&look);
	%end;

 /** IF MACRO VARIABLE ITT HAS A FOLLOW UP TIME SPECIFIED, 
	 THEN CREATE BOTH ITT and as TREATED EVENT INDICATOR and FOLLOW UP TIME
	 OTHERWISE, CREATE ONLY as TREATED INDICATOR and FOLLOW UP TIME
	 START FOLLOW UP TIME AFTER BLACKOUT PERIOD
	**/

%macro Followup_ITT_AST();

	PROC SORT DATA = _QUERYEVENTFILE (rename = (group = studyclass)
									  keep = group blackoutper)
					out = blackout nodupkey ; 
	by studyclass blackoutper;
	RUN; 

%global sasdate_itt sasdate_at a_type queryto;	

%if &ITT ne %str( ) %then %do;	

/* 	ASSUMES THAT EXPEXTPERIOD IN QUERYFILE 
	HAS EXTENDED TREATMENT PERIOD BY &ITT DAYS */

	%let a_type = ITT;
	DATA _null_ ;
	call symput('QUERYTO',put(input("&enddate",mmddyy10.),best12.));
	RUN;

	%put &a_type &QUERYTO;

 	DATA NEWUSER&LOOK;
    merge &nz_or_work.NEWUSER&LOOK (in = newuser) blackout;
	by studyclass;
	if newuser = 1;
		sasdate_ITT = indexdt+&ITT. -1 + blackoutper;
		endfollowup_itt =min(SASDATE_ITT, deathdt, discharged_dead, minenr_end, eventdt, &queryto); 
		FORMAT endfollowup_itt date9.;
		followuptime_itt = endfollowup_itt-indexdt+1 - blackoutper;
		event_itt = 0;
		IF endfollowup_itt = eventdt THEN event_itt = 1;
		if event_itt = 0 then eventdt = .;
     	LABEL eventdt = "DATE OF EVENT";
		KEEP patid indexdt studyclass exposure age_cat age sex race eventdt
      	 	 event_itt followuptime_itt 
      		 &covardxlist &covarrxlist &covarpxlist;
	RUN; 

%end;

%if &ITT = %str( ) %then %do;

/* ASSUMES THAT EXPEXTPERIOD IN QUERYFILE REFLECTS 
	ADDITIONAL DAYS TO CONSIDER PATIENT as TREATED */
	%let a_type = ASTREATED;
	DATA _null_ ;
	call symput('QUERYTO',put(input("&enddate",mmddyy10.),best12.));
	RUN;
	%put &a_type &QUERYTO;

	DATA NEWUSER&LOOK;
    merge &nz_or_work.NEWUSER&LOOK (in = newuser) blackout;
	by studyclass;
	if newuser = 1;
		endfollowup_at =min(episodeenddt, deathdt, discharged_dead, minenr_end, eventdt, &queryto); 
		FORMAT endfollowup_at date9.;

		followuptime_astreated = endfollowup_at-indexdt+1-blackoutper;
		event_astreated = 0;
		IF endfollowup_at = eventdt THEN event_astreated = 1;
     	LABEL eventdt = "DATE OF EVENT";
		if event_astreated = 0 then eventdt = .;
		KEEP patid indexdt studyclass exposure age_cat age sex race eventdt
      	 	 event_astreated followuptime_astreated
      		 &covardxlist &covarrxlist &covarpxlist;

	RUN; 

%end;
%mend;
%Followup_ITT_AST();

/** CREATE TIME VARIABLE - MONITORING PERIOD IN WHICH NEW USER WAS FIRST IDENTIFIED **/

      PROC SORT DATA = NEWUSER&LOOK; by patid indexdt; RUN;

      %MACRO TIME();

	  %if %sysfunc(exist(dplocal.&DPID.&SITEID._&REQUESTID._matched_&prior.)) = 1 %then %do;

      DATA FIRSTNEWUSER&LOOK;
      RETAIN patid indexdt studyclass exposure age_cat sex race
      event_&a_type followuptime_&a_type
      &covardxlist &covarrxlist &covarpxlist;
      MERGE dplocal.&DPID.&SITEID._&REQUESTID._matched_&prior. (in = p keep = patid indexdt time) 
			NEWUSER&LOOK (in = n);
      by patid indexdt;
	  if p = 0 then time = &look;
      RUN;

      %END;
      %IF %sysfunc(exist(dplocal.&DPID.&SITEID._&REQUESTID._matched_&prior.)) = 0 %THEN %DO;

      DATA FIRSTNEWUSER&LOOK;
      RETAIN patid indexdt studyclass exposure age_cat sex race
      event_&a_type followuptime_&a_type
      &covardxlist &covarrxlist &covarpxlist;
      SET NEWUSER&LOOK;
      TIME=&LOOK;
      RUN;

      %END;
      %MEND;
      %TIME();

/**	IF IDENTIFIED AS NEW USER MULTIPLE TIMES, KEEP THE FIRST    **/
	 PROC SORT DATA = FIRSTNEWUSER&LOOK; by patid indexdt time; RUN;
	 PROC SORT DATA = FIRSTNEWUSER&LOOK nodupkey; by patid; RUN;

/** SAVE DATASET OF NEW USERS TO DPLOCAL **/
     DATA dplocal.NEWUSER&LOOK;
	 SET FIRSTNEWUSER&LOOK; 
	 RUN;

/* IF NO NEW USERS IDENTIFIED DURING THIS MOST RECENT MONITORING PERIOD THEN SKIP:
	 creation of healthcare utilization variables
	 comorbid scores
	 hdPS
	 matching																
   IF NEW USERS ARE IDENTIFIED IN MONITORING PERIOD FOR ONLY 1 EXPOSURE GROUP THEN SKIP:
	 hdPS
	 matching																		**/

%macro hdPSMatch();

%if &numNewPatients>0 %then %do;

	     DATA &nz_or_work.TEMP2 &bulkload;
	     SET dplocal.NEWUSER&LOOK;
	     RUN;

	/*************************************************************************
	 CREATE COMBINED CHARLSON-ELIXHAUSER SCORE and UTILIZATION VARIABLES
	**************************************************************************/
	PROC SQL;

	     create table &nz_or_work.comorbid &bulkload as
	     select * from infolder.comorbidscore;

	     create table &nz_or_work.comorbid2 &bulkload as 
	     select distinct patid, disease, Weight as COMORBIDSCORE
	     from &nz_or_work.diag as D, &nz_or_work.comorbid as s
	     where (s.icd3 = d.D3 and d.D3 ne "")  
			or (s.icd4 = d.D4 and d.D4 ne "")
			or (s.icd5 = d.D5 and d.D5 ne "");

	     create table &nz_or_work.comorbid3 &bulkload as 
	     select patid, sum(COMORBIDSCORE) as cscore
		 from &nz_or_work.comorbid2
	 	 group by patid;

	 /********************************************************************
	  GET COUNT - # DAYS WITH ENCOUNTERS; # UNIQUE RX CLASS WITHIN
	  COVARIATE ASSESSMENT WINDOW
	 *********************************************************************/

	     create table &nz_or_work.NUME &bulkload AS
	     select t.patid, enctype, count(*) as numencounter
	     from (
	     	select distinct patid, ADATE, enctype
	     	from indata.&ENCTABLE 
			where patid in (select patid from &nz_or_work.TEMP2)
	     ) as A
		 inner join &nz_or_work.TEMP2 as t
		 on a.patid = t.patid
		 where INDEXDT-&covariatewindow<= ADATE and ADATE<INDEXDT
		 group by t.patid, enctype;
 
	QUIT;

	proc sort data = infolder.&drugclass out = ordered_drugs; by ndc generic; run;

	data uniquegeneric;
		set ordered_drugs;
		by ndc;
		length g $92.;
		retain g;
		if first.ndc then g = "";
		g = compress(g||generic);
		if last.ndc then output;
	run;

	PROC SQL;

		 create table &nz_or_work.DRUGCLASS &bulkload AS
     	 select D.NDC, CLASS, g, patid, RXDATE
         from uniquegeneric as D
         inner join &nz_or_work.rx as NDC
         on D.NDC = NDC.NDC;

  		 create table &nz_or_work.RCOUNT &bulkload AS
     	 select patid, COUNT(distinct g) as NUMGENERIC, count(*) as NUMRX
    	 from &nz_or_work.DRUGCLASS
     	 group by patid;

    QUIT;

		data numIP numED numIS numAV numOA;
			set &nz_or_work.NUME;
			if enctype = "IP" then output numIP;
			if enctype = "ED" then output numED;
			if enctype = "IS" then output numIS;
			if enctype = "AV" then output numAV;
			if enctype = "OA" then output numOA;
		run;

	PROC SQL;

	     create table &nz_or_work.TEMP3 &bulkload AS
	     select T.*, COALESCE(ip.numencounter, 0) as numIP, COALESCE(ed.numencounter, 0) as numED, COALESCE(is.numencounter, 0) as numIS, 
					 COALESCE(av.numencounter, 0) as numAV, COALESCE(oa.numencounter, 0) as numOA, 
					 COALESCE(numgeneric, 0) as numgeneric, COALESCE(numrx, 0) as numrx, COALESCE(CSCORE, 0) as COMORBIDSCORE, 
					 year(indexdt) as year
	     from &nz_or_work.TEMP2 as T
	     left outer join numIP as IP
	     on T.patid = IP.patid
		 left outer join numED as ED
	     on T.patid = ED.patid
		 left outer join numIS as IS
	     on T.patid = IS.patid
		 left outer join numAV as AV
	     on T.patid = AV.patid
		 left outer join numOA as OA
	     on T.patid = OA.patid
	     left outer join &nz_or_work.RCOUNT as R
	     on T.patid = R.patid
	     left outer join &nz_or_work.COMORBID3 as C
	     on T.patid = C.patid;
	QUIT;

%end; * if &numNewPatients >0;

/** IF <2 EXPOSURE GROUPS WITH NEW USERS IDENTIFIED THEN SKIP HDPS and MATCHING  **/
%IF &numNewExpgroups ge 2 %THEN %DO;

	 /** GET START TIME FOR CREATING HDPS COVARIATES **/

	      DATA _NULL_;
	        temp=DATETIME();
	        call symput('STHD',temp);
	        call symput('STDATEHD',put(datepart(temp),date9.));
	        call symput('STTIMEHD',put(timepart(temp),time4.));
	       RUN;

    PROC SQL;

 	    /** SET DEFAULT NUMBER OF HDPS COVARIATES TO BE THE SAMPLE SIZE (N)
	         OF NEW USERS OF A STUDY DRUG (WHICHEVER IS THE SMALLEST) **/
	     select count(*)
	     into :size1-:size2
	     from &nz_or_work.temp3
	     group by exposure;

	QUIT;

	PROC SQL;

	 	 /************************************************************************
	 	 CREATE SEPARATE PROCEDURES DATASETS BY CODETYPE
	 	 ************************************************************************/
		 create table &nz_or_work.cpt &bulkload as 
		 select * from
		 &nz_or_work.proc
		 where upcase(px_codetype) in("C4", "C2", "C3");

	 	 create table &nz_or_work.icd_p &bulkload as 
		 select * from
		 &nz_or_work.proc
		 where upcase(px_codetype) in("09", "10", "11");

	 	 create table &nz_or_work.hcpcs &bulkload as 
		 select * from
		 &nz_or_work.proc
		 where upcase(px_codetype) in("HC", "H3");

	QUIT;

	/** RUN HDPS MACROS **/

		%macro initHDPS();

			%global default_service default_n default_rank default_outcome_zero;

			%if &health_service_intensity = %str( ) %then %do;
			%let default_service = 1; %end;
			%else %let default_service = &health_service_intensity;

			%if &covariates_considered =%str(  ) %then %do;
			%let default_n = 100; %end;
			%else %let default_n = &covariates_considered;

			%if &covariates_selected=%str( ) %then %do;
    		%let newusernum = 200;
    		%end;
    		%else %let newusernum = &covariates_selected;
			%let min=%sysfunc(min(&size1.,&size2.));
		    %if &newusernum.>&min. %then %let default_k=&min.; 
    		%else %let default_k=&newusernum.;

			%if &ranking=%str(  ) %then %do;
			%let default_rank = EXP_ASSOC; %end;
			%else %let default_rank = &ranking;

			%if &zero_cell_corr=%str(  ) %then %do;
			%let default_outcome_zero = 1; %end;
			%else %let default_outcome_zero = &zero_cell_corr;

			/** RUN HDPS MACROS **/

			%InitHDMacros (
			 var_patient_id  =patid,
			 var_exposure = exposure,
			 var_outcome = event_&A_type,
			 vars_demographic = SEX AGE_CAT,
			 vars_force_categorical = SEX AGE_CAT TIME YEAR &covardxlist &covarrxlist &covarpxlist ,
			 vars_predefined = 	&covardxlist &covarrxlist &covarpxlist TIME  comorbidscore numIP numED numIS numAV numOA numRX numGENERIC,
			 infer_service_intensity_vars = &default_service,
			 top_n = &default_n,
			 k = &default_k,
			 ranking_method = &default_rank,
			 outcome_zero_cell_corr = &default_outcome_zero,
			 path_temp_dir = %QUOTE(&TEMPHDPS),
			 path_jar_file =%QUOTE(&jar),
			 
			 outcome_model_deciles  = 0,
			 outcome_model_matched  = 0,
			 score_type_1= 0,
			 score_type_2= 0,
			 score_type_3= 1,
			 score_type_4= 1,
			 score_type_5= 1,
			 input_cohort = &nz_or_work.temp3,
			 input_dim1= &nz_or_work.DIAG DX,
			 input_dim2= &nz_or_work.drugclass g,
			 input_dim3= &nz_or_work.cpt PX,
			 input_dim4= &nz_or_work.icd_p PX,
			 input_dim5= &nz_or_work.hcpcs PX,
			 output_cohort = SCORED_COHORT,
			 result_estimates = estimates,
			 result_diagnostic = variable_info);
			 %DoHDVariableSelection;
			 %EstimateHDPS(dataset_with_ps);
			 %RunOutcomeModels(dataset_with_ps, ps);

			%mend;
		%initHDPS();

		/** TIMING TO CREATE HDPS COVARIATES **/
	        DATA _NULL_;
	        temp=DATETIME();
	        seconds=temp-&STHD.;
	        hours=int(seconds/3600);
	        minutes=int((seconds-hours*3600)/60);
	        seconds2=int((seconds-hours*3600-minutes*60));
	        call symput('STOP',temp);
	        call symput('hours',put(hours,4.0));
	        call symput('minutes',put(minutes,2.0));
	        call symput('seconds',put(seconds2,2.0));
	        RUN;

			%global HDPStime;
	        %let HDPStime = &hours. h &minutes. m &seconds. s ;

		/*********************************************************************
		 OUTPUT FULL DATASETS TO DPLOCAL
		 *********************************************************************/

		/** LABEL PREDEFINED COVARIATES **/
		%macro LABELVAR();
		%IF &nocovarcondition ne 1 %THEN %DO; 
			DATA DATASET_WITH_PS; 
			SET DATASET_WITH_PS;
			%do i = 1 %to &NUMC;
			LABEL COVAR&i = "&&COVAR&i";
			%end;
			RUN;
		%END;
		%IF &nocovarrx ne 1 %THEN %DO;
			DATA DATASET_WITH_PS; 
			SET DATASET_WITH_PS;
			%do i = 1 %to &NUMR;
			LABEL COVMED&i = "&&COVMED&i";
			%end;
			RUN;
		%END;
		%IF &nocovarproc ne 1 %THEN %DO;
			DATA DATASET_WITH_PS; 
			SET DATASET_WITH_PS;
			%do i = 1 %to &NUMP;
			LABEL COVPROC&i = "&&COVPROC&i";
			%end;
			RUN;
		%END;
		%MEND; 
		%LABELVAR();

		DATA dplocal.&DPID.&SITEID._&REQUESTID._scores_&look.;
		SET DATASET_WITH_PS 
			(RENAME = (PS3 = PREDEFINEDPS) 
			 RENAME = (PS4 = HDPSPREDEFINED) 
			 RENAME = (PS5 = HDPSONLY));
		by patid;
		LABEL TIME = "INTERIM ANALYSIS NUMBER";
		RUN;

		DATA dplocal.&DPID.&SITEID._&REQUESTID._estimates_&look.;
		SET ESTIMATES;
		length PSTYPE $32.;
		IF MODEL_NUMBER = 3 THEN PSTYPE = "PREDEFINED";
		IF MODEL_NUMBER = 4 THEN PSTYPE = "HDPS and PREDEFINED";
		IF MODEL_NUMBER = 5 THEN PSTYPE = "HDPS ONLY"; 
		IF PSTYPE ne (" ");
		KEEP PSTYPE C_STAT SCORE_MODEL_CONVERGED;
		RUN;

		PROC SQL;
		select score_model_converged into :c1-:c3
		from  dplocal.&DPID.&SITEID._&REQUESTID._estimates_&look.;
		QUIT;
		%put &c1 &c2 &c3;

		DATA dplocal.DP_&REQUESTID._VARINFO_&LOOK;
		SET VARIABLE_INFO_ALL_VARS;
		RUN;

	 /** GET START TIME FOR MATCHING **/

	      DATA _NULL_;
	        temp=DATETIME();
	        call symput('STMATCH',temp);
	        call symput('STDATEM',put(datepart(temp),date9.));
	        call symput('STTIMEM',put(timepart(temp),time4.));
	       RUN;

	/***************************************************************************/
	/** MATCH WITHIN MONITORING PERIOD										  **/
	/** KEEP PRIOR MATCHES                                      			  **/
	/***************************************************************************/
	/** CLEAN UP **/
	PROC DATASETS LIBRARY = WORK nolist kill; QUIT; RUN; 
	RUN;

	/** SELECT PATIENTS IDENTIFIED IN MOST RECENT MONITORING PERIOD **/

		DATA TEMP&look;
		SET dplocal.&DPID.&SITEID._&REQUESTID._scores_&look.;
		WHERE TIME = &look;
		RUN;

	/** MATCH WITHIN STRATA OF QUARTER USING THREE CALIPERS **/

	%MACRO STRATAMATCH(PSTYPE = , converged= );
	%if &converged = 1 %then %do;
	
		%DO variable = 0 %to 1;

			%if &variable = 0 %then %let v = 1;
			%if &variable = 1 %then %let v = 100;

		 		%DO r = 1 %to 3;
		 			%IF &r = 1 %THEN %LET caliper = .01;
		 			%IF &r = 2 %THEN %LET caliper = .025;
		 			%IF &r = 3 %THEN %LET caliper = .05;

					%match_NearestNeighborMatch(
					in_dataset         =TEMP&look,
					out_dataset        =MATCHCALIPER&v._&r,
					working_directory  =%QUOTE(&temphdps),
					var_patient_id     =patid,
					var_exposure       = exposure,
					var_ps             = &PSTYPE,
					exp_groups         = 1 0,
					caliper            = &caliper,
					ratio              = &v,
					fixed_ratio        = 0);

					/** IF AT LEAST ONE MATCH FOUND THEN DO **/
					%if %sysfunc(exist(MATCHCALIPER&v._&r.)) %then %do;
						DATA &PSTYPE.MATCHCALIPER&v._&r.;
						SET MATCHCALIPER&v._&r.;
						length &PSTYPE.MATCHID&v._&r.$50.;
						&PSTYPE.MATCHID&v._&r.  = COMPRESS(TIME||SET_NUM);
						LABEL &PSTYPE.MATCHID&v._&r.= "&PSTYPE NEAREST NEIGHBOR CALIPER &caliper ratio &v.";
						KEEP patid &PSTYPE.MATCHID&v._&r.;
						RUN;    
					%end;
					
					/** IF NO MATCHES FOUND THEN DO **/
					%else %do;
						DATA &PSTYPE.MATCHCALIPER&v._&r.;
						SET TEMP&look (KEEP =patid);
						length &PSTYPE.MATCHID&v._&r.$50.;
						&PSTYPE.MATCHID&v._&r.  = " ";
						LABEL &PSTYPE.MATCHID&v._&r.= "&PSTYPE NEAREST NEIGHBOR CALIPER &caliper ratio &v.";
						KEEP patid &PSTYPE.MATCHID&v._&r.;
						RUN; 
					%end;
				
			%END; * r;
		%END; * variable;

	%END; * if ps model converged;

	%if &converged ne 1 %then %do;
		%DO variable = 0 %to 1;

				%if &variable = 0 %then %let v = 1;
				%if &variable = 1 %then %let v = 100;

			 		%DO r = 1 %to 3;
			 			%IF &r = 1 %THEN %LET caliper = .01;
			 			%IF &r = 2 %THEN %LET caliper = .025;
			 			%IF &r = 3 %THEN %LET caliper = .05;

						DATA &PSTYPE.MATCHCALIPER&v._&r.;
						SET TEMP&look (KEEP =patid);
						length &PSTYPE.MATCHID&v._&r.$50.;
						&PSTYPE.MATCHID&v._&r.  = " ";
						LABEL &PSTYPE.MATCHID&v._&r.= "&PSTYPE NEAREST NEIGHBOR CALIPER &caliper ratio &v.";
						KEEP patid &PSTYPE.MATCHID&v._&r.;
						RUN; 
					%END; * r;

		%END; * variable;

	%END; * if ps model did not converge;
	%MEND;

	%MACRO MATCHING();

	/** KEEP PRIOR MATCHES ***************/

		%STRATAMATCH(PSTYPE = PREDEFINEDPS, converged = &c1);
		%STRATAMATCH(PSTYPE = HDPSPREDEFINED, converged = &c2);
		%STRATAMATCH(PSTYPE = HDPSONLY, converged = &c3);

		PROC SORT DATA = HDPSPREDEFINEDMATCHCALIPER1_1; by patid; RUN;
		PROC SORT DATA = HDPSPREDEFINEDMATCHCALIPER1_2; by patid; RUN;
		PROC SORT DATA = HDPSPREDEFINEDMATCHCALIPER1_3; by patid; RUN;
		PROC SORT DATA = PREDEFINEDPSMATCHCALIPER1_1; by patid; RUN;
		PROC SORT DATA = PREDEFINEDPSMATCHCALIPER1_2; by patid; RUN;
		PROC SORT DATA = PREDEFINEDPSMATCHCALIPER1_3; by patid; RUN;
		PROC SORT DATA = HDPSONLYMATCHCALIPER1_1; by patid; RUN;
		PROC SORT DATA = HDPSONLYMATCHCALIPER1_2; by patid; RUN;
		PROC SORT DATA = HDPSONLYMATCHCALIPER1_3; by patid; RUN;

		PROC SORT DATA = HDPSPREDEFINEDMATCHCALIPER100_1; by patid; RUN;
		PROC SORT DATA = HDPSPREDEFINEDMATCHCALIPER100_2; by patid; RUN;
		PROC SORT DATA = HDPSPREDEFINEDMATCHCALIPER100_3; by patid; RUN;
		PROC SORT DATA = PREDEFINEDPSMATCHCALIPER100_1; by patid; RUN;
		PROC SORT DATA = PREDEFINEDPSMATCHCALIPER100_2; by patid; RUN;
		PROC SORT DATA = PREDEFINEDPSMATCHCALIPER100_3; by patid; RUN;
		PROC SORT DATA = HDPSONLYMATCHCALIPER100_1; by patid; RUN;
		PROC SORT DATA = HDPSONLYMATCHCALIPER100_2; by patid; RUN;
		PROC SORT DATA = HDPSONLYMATCHCALIPER100_3; by patid; RUN;

		PROC SORT DATA = dplocal.&DPID.&SITEID._&REQUESTID._scores_&look.; by patid; RUN;

		%if &look = 1 or %sysfunc(exist(dplocal.&DPID.&SITEID._&REQUESTID._matched_&PRIOR)) ne 1 %then %do;
		DATA dplocal.&DPID.&SITEID._&REQUESTID._matched_&LOOK.;
		MERGE HDPSPREDEFINEDMATCHCALIPER1_1  HDPSPREDEFINEDMATCHCALIPER1_2 HDPSPREDEFINEDMATCHCALIPER1_3
		      PREDEFINEDPSMATCHCALIPER1_1 PREDEFINEDPSMATCHCALIPER1_2 PREDEFINEDPSMATCHCALIPER1_3
			  HDPSONLYMATCHCALIPER1_1  HDPSONLYMATCHCALIPER1_2 HDPSONLYMATCHCALIPER1_3
			  HDPSPREDEFINEDMATCHCALIPER100_1  HDPSPREDEFINEDMATCHCALIPER100_2 HDPSPREDEFINEDMATCHCALIPER100_3
		      PREDEFINEDPSMATCHCALIPER100_1 PREDEFINEDPSMATCHCALIPER100_2 PREDEFINEDPSMATCHCALIPER100_3
			  HDPSONLYMATCHCALIPER100_1  HDPSONLYMATCHCALIPER100_2 HDPSONLYMATCHCALIPER100_3
		      dplocal.&DPID.&SITEID._&REQUESTID._scores_&look.;
		by patid;
		RUN; 
		%END;
		%else %if %sysfunc(exist(dplocal.&DPID.&SITEID._&REQUESTID._matched_&PRIOR)) %then %do;
		DATA temp_matched;
		MERGE temp&look (keep = patid)
			  HDPSPREDEFINEDMATCHCALIPER1_1  HDPSPREDEFINEDMATCHCALIPER1_2 HDPSPREDEFINEDMATCHCALIPER1_3
		      PREDEFINEDPSMATCHCALIPER1_1 PREDEFINEDPSMATCHCALIPER1_2 PREDEFINEDPSMATCHCALIPER1_3
			  HDPSONLYMATCHCALIPER1_1  HDPSONLYMATCHCALIPER1_2 HDPSONLYMATCHCALIPER1_3
			  HDPSPREDEFINEDMATCHCALIPER100_1  HDPSPREDEFINEDMATCHCALIPER100_2 HDPSPREDEFINEDMATCHCALIPER100_3
		      PREDEFINEDPSMATCHCALIPER100_1 PREDEFINEDPSMATCHCALIPER100_2 PREDEFINEDPSMATCHCALIPER100_3
			  HDPSONLYMATCHCALIPER100_1  HDPSONLYMATCHCALIPER100_2 HDPSONLYMATCHCALIPER100_3;
			  by patid;
		RUN;
		DATA temp_allmatched;
		SET temp_matched 
			dplocal.&DPID.&SITEID._&REQUESTID._matched_&PRIOR;
		RUN;
		PROC SORT DATA = temp_allmatched; by patid; RUN;
		DATA dplocal.&DPID.&SITEID._&REQUESTID._matched_&LOOK;
		MERGE 	temp_allmatched 
				dplocal.&DPID.&SITEID._&REQUESTID._scores_&look.;
		by patid;
		RUN;
		%END;  

	%MEND;
	%MATCHING();

	/** TIMING TO RUN MATCHING **/

        DATA _NULL_;
        temp=DATETIME();
        seconds=temp-&STMATCH.;
        hours=int(seconds/3600);
        minutes=int((seconds-hours*3600)/60);
        seconds2=int((seconds-hours*3600-minutes*60));
        call symput('STOP',temp);
        call symput('hours',put(hours,4.0));
        call symput('minutes',put(minutes,2.0));
        call symput('seconds',put(seconds2,2.0));
        RUN;

		%global matchingtime;
        %let matchingtime = &hours. h &minutes. m &seconds. s ;

%END; * if &numNewExpgroups ge 2;


%IF &numNewExpGroups<2 %THEN %DO;

		%if &look = 1 or %sysfunc(exist(dplocal.&DPID.&SITEID._&REQUESTID._matched_&PRIOR)) ne 1 %then %do;

			DATA dplocal.&DPID.&SITEID._&REQUESTID._scores_&look.;
				SET &nz_or_work.TEMP3;
				by patid;
				LABEL TIME = "INTERIM ANALYSIS NUMBER";
			RUN;

			DATA dplocal.&DPID.&SITEID._&REQUESTID._matched_&look.;
				RETAIN studyclass age_cat sex race EVENT_&a_type FOLLOWUPTIME_&a_type
				       &covardxlist &covarrxlist &covarpxlist;
				SET dplocal.&DPID.&SITEID._&REQUESTID._scores_&look.;
				length 	&matchid. $50.;
				
				array matchid(18) &matchid.;
				do i = 1 to 18;
					matchid(i) = " ";
				end;
				predefinedps = .;
				hdpspredefined = .;
				hdpsonly = .;

			RUN;

		%end;
		%if %sysfunc(exist(dplocal.&DPID.&SITEID._&REQUESTID._matched_&PRIOR)) %then %do;
			DATA dplocal.&DPID.&SITEID._&REQUESTID._scores_&look.;
				SET &nz_or_work.TEMP3;
				by patid;
				LABEL TIME = "INTERIM ANALYSIS NUMBER";
			RUN;

			DATA TEMP_matched
				set dplocal.&DPID.&SITEID._&REQUESTID._scores_&look.;
				where TIME = &look;
				length 	&matchid. $50.;
				array matchid(18) &matchid.;
				do i = 1 to 18;
					matchid(i) = " ";
				end;
			RUN;

			DATA temp_allmatched;
			SET temp_matched 
				dplocal.&DPID.&SITEID._&REQUESTID._matched_&PRIOR;
			RUN;
			PROC SORT DATA = temp_allmatched; by patid; RUN;
			DATA dplocal.&DPID.&SITEID._&REQUESTID._matched_&LOOK;
			MERGE 	temp_allmatched 
					dplocal.&DPID.&SITEID._&REQUESTID._scores_&look.;
			by patid;
			RUN;

		%end;  

%END; *&numexposuregroups<2;

%if &numNewPatients = 0 and %sysfunc(exist(dplocal.&DPID.&SITEID._&REQUESTID._matched_&prior.))%then %do;
	data dplocal.&DPID.&SITEID._&REQUESTID._matched_&look.;
	merge dplocal.&DPID.&SITEID._&REQUESTID._matched_&prior. (drop = event_&a_type followuptime_&a_type)
		  dplocal.newuser&look (keep = patid event_&a_type followuptime_&a_type);
	by patid;
	run;
%end; *if numNewPatients = 0;

	/* Output dataset for MSOC without patient identifiers or covariates (other than subgroup indicators) */
			DATA msoc.&DPID.&SITEID._&REQUESTID._matched_&look.;
				RETAIN studyclass age_cat sex race EVENT_&a_type FOLLOWUPTIME_&a_type &matchid;
				SET dplocal.&DPID.&SITEID._&REQUESTID._matched_&LOOK;
				KEEP 	exposure studyclass age_cat sex race EVENT_&a_type FOLLOWUPTIME_&a_type eventdt
			    		PREDEFINEDPS HDPSPREDEFINED HDPSONLY 
						AGE COMORBIDSCORE TIME YEAR numIP numED numIS numAV numOA numRX numGeneric
			 			&matchid &subgroup_dx &subgroup_px &subgroup_rx;
			RUN;

%MEND; *hdPSmatch;
%HDPSMATCH();

/*********************************
 CLEAN UP
*********************************/
%cleanup();

/** GET RUN TIME **/

        DATA _NULL_;
        temp=DATETIME();
        seconds=temp-&start.;
        hours=int(seconds/3600);
        minutes=int((seconds-hours*3600)/60);
        seconds2=int((seconds-hours*3600-minutes*60));
        call symput('STOP',temp);
        call symput('hours',put(hours,4.0));
        call symput('minutes',put(minutes,2.0));
        call symput('seconds',put(seconds2,2.0));
        RUN;

        %put TOTAL RUN TIME was &hours. h &minutes. m &seconds. s;

        %global covariatetime;
        %let covariatetime =  &hours. h &minutes. m &seconds. s ;

        DATA TEMP;
		length ADJUSTMENTRUNTIME $150.;
		length PREDEFINEDRUNTIME $150.;
		length HDPSRUNTIME $150.;
		length MATCHRUNTIME $150.;
		length MONITORINGPERIOD $2.;
		length DATES $30.;
		DO i = 1 to 1;
		ADJUSTMENTRUNTIME = "&covariatetime ";
		PREDEFINEDRUNTIME = "&predefinedtime ";
		HDPSRUNTIME = "&HDPStime ";
		MATCHRUNTIME = "&matchingtime ";
		MONITORINGPERIOD = put(&LOOK, 2.);
		DATES = "&startfollowup. - &enddate.";
		OUTPUT;
		DROP i;
		END;
		RUN;

		DATA MSOC.&DPID.&SITEID._&REQUESTID._TIMING;
		RETAIN MONITORINGPERIOD DATES COHORTRUNTIME ADJUSTMENTRUNTIME HDPSRUNTIME MATCHRUNTIME;
		MERGE MSOC.&DPID.&SITEID._&REQUESTID._TIMING TEMP;
		BY MONITORINGPERIOD;
		RUN;

	proc printto log=log;
	RUN;

%MEND COVARIATEADJUSTMENT;



